@@ -16,11 +16,12 @@
 #
 
 from ocw import dataset as ds
+import ocw.utils as utils
 
 import datetime
 import numpy as np
 import numpy.ma as ma
-import scipy.interpolate
+from scipy.interpolate import griddata
 import scipy.ndimage
 from scipy.stats import rankdata
 from scipy.ndimage import map_coordinates
@@ -32,7 +33,7 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-def temporal_subset(month_start, month_end, target_dataset,
+def temporal_subset(target_dataset, month_start, month_end,
                     average_each_year=False):
     """ Temporally subset data given month_index.
 
@@ -53,8 +54,8 @@ def temporal_subset(month_start, month_end, target_dataset,
     """
 
     if month_start > month_end:
-        month_index = range(month_start, 13)
-        month_index.extend(range(1, month_end + 1))
+        month_index = list(range(month_start, 13))
+        month_index.extend(list(range(1, month_end + 1)))
     else:
         month_index = range(month_start, month_end + 1)
 
@@ -62,10 +63,8 @@ def temporal_subset(month_start, month_end, target_dataset,
     months = np.array([d.month for d in dates])
     time_index = []
     for m_value in month_index:
-        time_index = np.append(time_index, np.where(months == m_value)[0])
-    time_index = np.sort(time_index)
-
-    time_index = list(time_index)
+        time_index.extend(list(np.where(months == m_value)[0]))
+    time_index.sort()
 
     new_dataset = ds.Dataset(target_dataset.lats,
                              target_dataset.lons,
@@ -78,14 +77,19 @@ def temporal_subset(month_start, month_end, target_dataset,
     if average_each_year:
         nmonth = len(month_index)
         ntime = new_dataset.times.size
-        nyear = ntime / nmonth
+        nyear = ntime // nmonth
+        if ntime % nmonth != 0:
+            raise ValueError("Number of times in dataset ({}) does not "
+                             "divide evenly into {} year(s)."
+                             .format(ntime, nyear))
+
         averaged_time = []
         ny, nx = target_dataset.values.shape[1:]
         averaged_values = ma.zeros([nyear, ny, nx])
         for iyear in np.arange(nyear):
             # centered time index of the season between month_start and
             # month_end in each year
-            center_index = int(nmonth / 2) + iyear * nmonth
+            center_index = int(nmonth / 2 + iyear * nmonth)
             if nmonth == 1:
                 center_index = iyear
             averaged_time.append(new_dataset.times[center_index])
@@ -147,11 +151,10 @@ def temporal_rebin_with_time_index(target_dataset, nt_average):
     """
     nt = target_dataset.times.size
     if nt % nt_average != 0:
-        msg = ('Warning: length of time dimension must '
-               'be a multiple of nt_average')
-        print msg
+        logger.warning('Length of time dimension must '
+                       'be a multiple of nt_average')
     # nt2 is the length of time dimension in the rebinned dataset
-    nt2 = nt / nt_average
+    nt2 = nt // nt_average
     binned_dates = target_dataset.times[np.arange(nt2) * nt_average]
     binned_values = ma.zeros(
         np.insert(target_dataset.values.shape[1:], 0, nt2))
@@ -213,12 +216,13 @@ def spatial_regrid(target_dataset, new_latitudes, new_longitudes,
     ny_old, nx_old = lats.shape
     ny_new, nx_new = new_lats.shape
 
+    for iy in np.arange(ny_old):
+        if not all(x < y for x, y in zip(lons[iy, :], lons[iy, 1:])):
+            lons[iy, :][lons[iy, :] < 0] = lons[iy, :][lons[iy, :] < 0] + 360.
+
     # Make masked array of shape (times, new_latitudes,new_longitudes)
     new_values = ma.zeros([len(target_dataset.times),
-                           ny_new, nx_new])
-    # Make masked array of shape (times, new_latitudes,new_longitudes)
-    new_values = ma.zeros([len(target_dataset.times),
-                           ny_new, nx_new])
+                           ny_new, nx_new])+1.e+20
 
     # Boundary vertices of target_dataset
     vertices = []
@@ -242,79 +246,54 @@ def spatial_regrid(target_dataset, new_latitudes, new_longitudes,
         for ix in np.arange(nx_old)[::-1]:
             vertices.append([lons[0, ix], lats[0, ix]])
     path = Path(vertices)
-
-    # Convert new_lats and new_lons to float indices
-    new_lons_indices = np.zeros(new_lons.shape)
-    new_lats_indices = np.zeros(new_lats.shape)
+    new_xy_mask = np.ones(new_lats.shape)
 
     for iy in np.arange(ny_new):
         for ix in np.arange(nx_new):
             if path.contains_point([new_lons[iy, ix],
                                     new_lats[iy, ix]]) or not boundary_check:
-                if regular_grid:
-                    mn = lats.min()
-                    mx = lats.max()
-                    new_lats_indices[iy, ix] = (
-                        ny_old - 1.) * (new_lats[iy, ix] - mn)/ (mx - mn)
-                    mn = lons.min()
-                    mx = lons.max()
-                    new_lons_indices[iy, ix] = (
-                        nx_old - 1.) * (new_lons[iy, ix] - mn) / (mx - mn)
-                else:
-                    distance_from_original_grids = (
-                        (lons - new_lons[iy, ix])**2. +
-                        (lats - new_lats[iy, ix])**2.)**0.5
-                    if np.min(distance_from_original_grids) == 0.:
-                        new_lats_indices[iy, ix], new_lons_indices[
-                            iy, ix] = np.where(
-                                distance_from_original_grids == 0)
-                    else:
-                        distance_rank = rankdata(
-                            distance_from_original_grids.flatten(),
-                            method='ordinal').reshape(lats.shape)
-                        # the nearest grid point's indices
-                        iy1, ix1 = np.where(distance_rank == 1)
-                        # point [iy2, ix] is diagonally across from [iy1, ix1]
-                        iy2, ix2 = np.where(distance_rank == 4)
-                        dist1 = distance_from_original_grids[iy1, ix1]
-                        dist2 = distance_from_original_grids[iy2, ix2]
-                        new_lats_indices[iy, ix] = (
-                            dist1 * iy2 + dist2 * iy1) / (dist1 + dist2)
-                        new_lons_indices[iy, ix] = (
-                            dist1 * ix2 + dist2 * ix1) / (dist1 + dist2)
-            else:
-                new_lats_indices[iy, ix] = -999.
-                new_lats_indices[iy, ix] = -999.
-    new_lats_indices = ma.masked_less(new_lats_indices, 0.)
-    new_lons_indices = ma.masked_less(new_lons_indices, 0.)
-
+               new_xy_mask[iy, ix] = 0.
+            
+    new_index = np.where(new_xy_mask == 0.)
     # Regrid the data on each time slice
     for i in range(len(target_dataset.times)):
-        if len(target_dataset.times) == 1:
+        if len(target_dataset.times) == 1 and target_dataset.values.ndim == 2:
             values_original = ma.array(target_dataset.values)
         else:
             values_original = ma.array(target_dataset.values[i])
+        new_mask = np.copy(values_original.mask)
         for shift in (-1, 1):
             for axis in (0, 1):
                 q_shifted = np.roll(values_original, shift=shift, axis=axis)
-                idx = ~q_shifted.mask * values_original.mask
-                values_original.data[idx] = q_shifted[idx]
-        new_values[i] = map_coordinates(values_original,
-                                        [new_lats_indices.flatten(),
-                                         new_lons_indices.flatten()],
-                                        order=1).reshape(new_lats.shape)
-        new_values[i] = ma.array(new_values[i], mask=new_lats_indices.mask)
+                if (np.where((values_original.mask == True) & (q_shifted.mask == False)))[0].size !=0:
+                    index1 =np.where((values_original.mask == True) & (q_shifted.mask == False))
+                    n_indices = len(index1[0])
+                    values_original.data[index1] = q_shifted[index1]
+                    new_mask[index1] = np.repeat(False, n_indices)
+                    mask_index = np.where(~new_mask)
+        if new_mask.size != 1:
+            mask_index = np.where(~new_mask)
+        else:
+            mask_index = np.where(~np.isnan(values_original))
+        new_values_temp = griddata((lons[mask_index], lats[mask_index]), values_original[mask_index],
+                              (new_lons[new_index],
+                               new_lats[new_index]),
+                               method='linear')
         # Make a masking map using nearest neighbour interpolation -use this to
         # determine locations with MDI and mask these
         qmdi = np.zeros_like(values_original)
-        qmdi[values_original.mask == True] = 1.
-        qmdi[values_original.mask == False] = 0.
-        qmdi_r = map_coordinates(qmdi, [new_lats_indices.flatten(
-        ), new_lons_indices.flatten()], order=1).reshape(new_lats.shape)
-        mdimask = (qmdi_r != 0.0)
-
+        values_true_indices = np.where(values_original.mask == True)
+        values_false_indices = np.where(values_original.mask == False)
+        qmdi[values_true_indices] = 1.
+        qmdi[values_false_indices] = 0.
+        qmdi_r = griddata((lons.flatten(), lats.flatten()), qmdi.flatten(), 
+                             (new_lons[new_index],
+                              new_lats[new_index]),
+                              method='nearest')
+        new_values_temp = ma.masked_where(qmdi_r != 0.0, new_values_temp)
         # Combine missing data mask, with outside domain mask define above.
-        new_values[i].mask = np.logical_or(mdimask, new_values[i].mask)
+        new_values[i, new_index[0], new_index[1]] = new_values_temp[:]
+        new_values[i,:] = ma.masked_equal(new_values[i,:], 1.e+20)
 
     # TODO:
     # This will call down to the _congrid() function and the lat and lon
@@ -362,7 +341,7 @@ def ensemble(datasets):
     return ensemble_dataset
 
 
-def subset(subregion, target_dataset, subregion_name=None):
+def subset(target_dataset, subregion, subregion_name=None, extract=True, user_mask_values=[1]):
     '''Subset given dataset(s) with subregion information
 
     :param subregion: The Bounds with which to subset the target Dataset.
@@ -374,6 +353,12 @@ def subset(subregion, target_dataset, subregion_name=None):
     :param subregion_name: The subset-ed Dataset name
     :type subregion_name: :mod:`string`
 
+    :param extract: If False, the dataset inside regions will be masked.
+    :type extract: :mod:`boolean`
+
+    :param user_mask_value: grid points where mask_variable == user_mask_value will be extracted or masked .
+    :type user_mask_value: :mod:`int`
+
     :returns: The subset-ed Dataset object
     :rtype: :class:`dataset.Dataset`
 
@@ -381,89 +366,131 @@ def subset(subregion, target_dataset, subregion_name=None):
     '''
 
     if not subregion.start:
-        subregion.start = target_dataset.times[0]
-        subregion.end = target_dataset.times[-1]
-
-    # Ensure that the subregion information is well formed
-    _are_bounds_contained_by_dataset(subregion, target_dataset)
+        time_start = target_dataset.times[0]
+        time_end = target_dataset.times[-1]
+    else:
+        time_start = subregion.start
+        time_end = subregion.end
 
     if not subregion_name:
         subregion_name = target_dataset.name
 
-    if target_dataset.lats.ndim == 2 and target_dataset.lons.ndim == 2:
-        start_time_index = np.where(
-            target_dataset.times == subregion.start)[0][0]
-        end_time_index = np.where(target_dataset.times == subregion.end)[0][0]
-        target_dataset = temporal_slice(
-            start_time_index, end_time_index, target_dataset)
-        nt, ny, nx = target_dataset.values.shape
-        y_index, x_index = np.where(
-            (target_dataset.lats >= subregion.lat_max) | (
-                target_dataset.lats <= subregion.lat_min) |
-            (target_dataset.lons >= subregion.lon_max) | (
-                target_dataset.lons <= subregion.lon_min))
-        for it in np.arange(nt):
-            target_dataset.values[it, y_index, x_index] = 1.e+20
-        target_dataset.values = ma.masked_equal(target_dataset.values, 1.e+20)
-        return target_dataset
+    if hasattr(subregion, 'lat_min'):
+        _are_bounds_contained_by_dataset(target_dataset, subregion)
 
-    elif target_dataset.lats.ndim == 1 and target_dataset.lons.ndim == 1:
-        # Get subregion indices into subregion data
-        dataset_slices = _get_subregion_slice_indices(subregion,
-                                                      target_dataset)
-        # Slice the values array with our calculated slice indices
-        if target_dataset.values.ndim == 2:
-            subset_values = ma.zeros([len(target_dataset.values[
-                dataset_slices["lat_start"]:dataset_slices["lat_end"]]),
-                len(target_dataset.values[
-                    dataset_slices["lon_start"]:dataset_slices["lon_end"]])])
+        if target_dataset.lats.ndim == 2 and target_dataset.lons.ndim == 2:
+            temporal_subset = temporal_slice(
+                target_dataset, time_start, time_end)
+            nt = temporal_subset.values.shape[0]
+            y_index, x_index = np.where(
+                (target_dataset.lats >= subregion.lat_max) | (
+                    target_dataset.lats <= subregion.lat_min) |
+                (target_dataset.lons >= subregion.lon_max) | (
+                    target_dataset.lons <= subregion.lon_min))
+            for it in np.arange(nt):
+                temporal_subset.values[it, y_index, x_index] = 1.e+20
+            new_values = ma.masked_equal(
+                temporal_subset.values, 1.e+20)
+            return ds.Dataset(
+                target_dataset.lats,
+                target_dataset.lons,
+                temporal_subset.times,
+                new_values,
+                variable=target_dataset.variable,
+                units=target_dataset.units,
+                name=subregion_name,
+                origin=target_dataset.origin)
 
-            subset_values = target_dataset.values[
-                dataset_slices["lat_start"]:dataset_slices["lat_end"] + 1,
-                dataset_slices["lon_start"]:dataset_slices["lon_end"] + 1]
-
-        elif target_dataset.values.ndim == 3:
-            subset_values = ma.zeros([len(target_dataset.values[
-                dataset_slices["time_start"]:dataset_slices["time_end"]]),
-                len(target_dataset.values[
-                    dataset_slices["lat_start"]:dataset_slices["lat_end"]]),
-                len(target_dataset.values[
-                    dataset_slices["lon_start"]:dataset_slices["lon_end"]])])
-
-            subset_values = target_dataset.values[
-                dataset_slices["time_start"]:dataset_slices["time_end"] + 1,
-                dataset_slices["lat_start"]:dataset_slices["lat_end"] + 1,
-                dataset_slices["lon_start"]:dataset_slices["lon_end"] + 1]
-
-        # Build new dataset with subset information
-        return ds.Dataset(
-            # Slice the lats array with our calculated slice indices
-            target_dataset.lats[dataset_slices["lat_start"]:
-                                dataset_slices["lat_end"] + 1],
-            # Slice the lons array with our calculated slice indices
-            target_dataset.lons[dataset_slices["lon_start"]:
-                                dataset_slices["lon_end"] + 1],
-            # Slice the times array with our calculated slice indices
-            target_dataset.times[dataset_slices["time_start"]:
-                                 dataset_slices["time_end"] + 1],
+        elif target_dataset.lats.ndim == 1 and target_dataset.lons.ndim == 1:
+            # Get subregion indices into subregion data
+            dataset_slices = _get_subregion_slice_indices(target_dataset,
+                                                          subregion)
             # Slice the values array with our calculated slice indices
+            if target_dataset.values.ndim == 2:
+                subset_values = ma.zeros([len(target_dataset.values[
+                    dataset_slices["lat_start"]:dataset_slices["lat_end"]]),
+                    len(target_dataset.values[
+                        dataset_slices["lon_start"]:dataset_slices["lon_end"]])])
+
+                subset_values = target_dataset.values[
+                    dataset_slices["lat_start"]:dataset_slices["lat_end"] + 1,
+                    dataset_slices["lon_start"]:dataset_slices["lon_end"] + 1]
+
+            elif target_dataset.values.ndim == 3:
+                subset_values = ma.zeros([len(target_dataset.values[
+                    dataset_slices["time_start"]:dataset_slices["time_end"]]),
+                    len(target_dataset.values[
+                        dataset_slices["lat_start"]:dataset_slices["lat_end"]]),
+                    len(target_dataset.values[
+                        dataset_slices["lon_start"]:dataset_slices["lon_end"]])])
+
+                subset_values = target_dataset.values[
+                    dataset_slices["time_start"]:dataset_slices["time_end"] + 1,
+                    dataset_slices["lat_start"]:dataset_slices["lat_end"] + 1,
+                    dataset_slices["lon_start"]:dataset_slices["lon_end"] + 1]
+            # Build new dataset with subset information
+            return ds.Dataset(
+                # Slice the lats array with our calculated slice indices
+                target_dataset.lats[dataset_slices["lat_start"]:
+                                    dataset_slices["lat_end"] + 1],
+                # Slice the lons array with our calculated slice indices
+                target_dataset.lons[dataset_slices["lon_start"]:
+                                    dataset_slices["lon_end"] + 1],
+                # Slice the times array with our calculated slice indices
+                target_dataset.times[dataset_slices["time_start"]:
+                                     dataset_slices["time_end"] + 1],
+                # Slice the values array with our calculated slice indices
+                subset_values,
+                variable=target_dataset.variable,
+                units=target_dataset.units,
+                name=subregion_name,
+                origin=target_dataset.origin
+            )
+
+    if subregion.boundary_type == 'us_states' or subregion.boundary_type == 'countries':
+        temporal_subset = temporal_slice(
+            target_dataset, time_start, time_end)
+        spatial_mask = utils.mask_using_shapefile_info(target_dataset.lons, target_dataset.lats,
+                                                       subregion.masked_regions, extract=extract)
+        subset_values = utils.propagate_spatial_mask_over_time(
+            temporal_subset.values, mask=spatial_mask)
+        return ds.Dataset(
+            target_dataset.lats,
+            target_dataset.lons,
+            temporal_subset.times,
             subset_values,
             variable=target_dataset.variable,
             units=target_dataset.units,
             name=subregion_name,
-            origin=target_dataset.origin
-        )
+            origin=target_dataset.origin)
 
+    if subregion.boundary_type == 'user':
+        temporal_subset = temporal_slice(
+            target_dataset, time_start, time_end)
+        spatial_mask = utils.regrid_spatial_mask(target_dataset.lons, target_dataset.lats,
+                                                 subregion.mask_longitude, subregion.mask_latitude, subregion.mask_variable,
+                                                 user_mask_values, extract=extract)
+        subset_values = utils.propagate_spatial_mask_over_time(
+            temporal_subset.values, mask=spatial_mask)
+        return ds.Dataset(
+            target_dataset.lats,
+            target_dataset.lons,
+            temporal_subset.times,
+            subset_values,
+            variable=target_dataset.variable,
+            units=target_dataset.units,
+            name=subregion_name,
+            origin=target_dataset.origin)
 
-def temporal_slice(start_time_index, end_time_index, target_dataset):
+def temporal_slice(target_dataset, start_time, end_time):
     '''Temporally slice given dataset(s) with subregion information. This does not
     spatially subset the target_Dataset
 
-    :param start_time_index: time index of the start time
-    :type start_time_index: :class:'int'
+    :param start_time: start time
+    :type start_time: :class:'int'
 
-    :param end_time_index: time index of the end time
-    :type end_time_index: :class:'int'
+    :param end_time: end time
+    :type end_time: :class:'datetime.datetime'
 
     :param target_dataset: The Dataset object to subset.
     :type target_dataset: :class:`dataset.Dataset`
@@ -473,19 +500,24 @@ def temporal_slice(start_time_index, end_time_index, target_dataset):
 
     :raises: ValueError
     '''
-    start_date = target_dataset.times[start_time_index]
-    end_date = target_dataset.times[end_time_index]
-    timeStart = min(np.nonzero(target_dataset.times >= start_date)[0])
-    timeEnd = max(np.nonzero(target_dataset.times <= end_date)[0])
-    target_dataset.times = target_dataset.times[timeStart:timeEnd + 1]
-    target_dataset.values = target_dataset.values[timeStart:timeEnd + 1, :]
+    start_time_index = np.where(
+        target_dataset.times >= start_time)[0][0]
+    end_time_index = np.where(
+        target_dataset.times <= end_time)[0][-1]
+    new_times = target_dataset.times[start_time_index:end_time_index + 1]
+    new_values = target_dataset.values[start_time_index:end_time_index + 1, :]
 
-    return target_dataset
+    return ds.Dataset(
+        target_dataset.lats,
+        target_dataset.lons,
+        new_times,
+        new_values,
+        variable=target_dataset.variable,
+        units=target_dataset.units,
+        origin=target_dataset.origin)
 
-
-def safe_subset(subregion, target_dataset, subregion_name=None):
+def safe_subset(target_dataset, subregion, subregion_name=None):
     '''Safely subset given dataset with subregion information
-
     A standard subset requires that the provided subregion be entirely
     contained within the datasets bounds. `safe_subset` returns the
     overlap of the subregion and dataset without returning an error.
@@ -504,19 +536,19 @@ def safe_subset(subregion, target_dataset, subregion_name=None):
     '''
 
     lat_min, lat_max, lon_min, lon_max = target_dataset.spatial_boundaries()
-    start, end = target_dataset.time_range()
+    start, end = target_dataset.temporal_boundaries()
+    if subregion.lat_min:
+        if subregion.lat_min < lat_min:
+            subregion.lat_min = lat_min
 
-    if subregion.lat_min < lat_min:
-        subregion.lat_min = lat_min
+        if subregion.lat_max > lat_max:
+            subregion.lat_max = lat_max
 
-    if subregion.lat_max > lat_max:
-        subregion.lat_max = lat_max
+        if subregion.lon_min < lon_min:
+            subregion.lon_min = lon_min
 
-    if subregion.lon_min < lon_min:
-        subregion.lon_min = lon_min
-
-    if subregion.lon_max > lon_max:
-        subregion.lon_max = lon_max
+        if subregion.lon_max > lon_max:
+            subregion.lon_max = lon_max
 
     if subregion.start:
         if subregion.start < start:
@@ -526,7 +558,7 @@ def safe_subset(subregion, target_dataset, subregion_name=None):
         if subregion.end > end:
             subregion.end = end
 
-    return subset(subregion, target_dataset, subregion_name)
+    return subset(target_dataset, subregion, subregion_name)
 
 
 def normalize_dataset_datetimes(dataset, timestep):
@@ -739,16 +771,17 @@ def water_flux_unit_conversion(dataset):
 
 
 def temperature_unit_conversion(dataset):
-    ''' Convert temperature units as necessary
-
+    ''' Convert temperature units as necessary \
     Automatically convert Celcius to Kelvin in the given dataset.
 
-    :param dataset: The dataset for which units should be updated.
+    :param dataset: The dataset for which units should be updated. \
     :type dataset; :class:`dataset.Dataset`
 
-    :returns: The dataset with (potentially) updated units.
+    :returns: The dataset with (potentially) updated units. \
     :rtype: :class:`dataset.Dataset`
+
     '''
+
     temperature_variables = ['temp', 'tas', 'tasmax', 'taxmin', 'T', 'tg']
     variable = dataset.variable.lower()
 
@@ -793,6 +826,7 @@ def _rcmes_normalize_datetimes(datetimes, timestep):
     :param timestep: The flag for how to normalize the datetimes.
     :type timestep: String
     """
+
     normalDatetimes = []
     if timestep.lower() == 'monthly':
         for inputDatetime in datetimes:
@@ -825,6 +859,7 @@ def mask_missing_data(dataset_array):
     the values at the grid point in all other datasets are masked.
     :param dataset_array: an array of OCW datasets
     '''
+
     mask_array = np.zeros(dataset_array[0].values.shape)
     for dataset in dataset_array:
         index = np.where(dataset.values.mask == True)
@@ -837,6 +872,32 @@ def mask_missing_data(dataset_array):
     return [masked_dataset for masked_dataset in masked_array]
 
 
+def deseasonalize_dataset(dataset):
+    '''Calculate daily climatology and subtract the climatology from
+    the input dataset
+
+    :param dataset: The dataset to convert.
+    :type dataset: :class:`dataset.Dataset`
+
+    :returns: A Dataset with values converted to new units.
+    :rtype: :class:`dataset.Dataset`
+    '''
+
+    days = [d.month * 100. + d.day for d in dataset.times]
+    days_sorted = np.unique(days)
+    ndays = days_sorted.size
+    nt, ny, nx = dataset.values.shape
+    values_clim = ma.zeros([ndays, ny, nx])
+    for iday, day in enumerate(days_sorted):
+        t_index = np.where(days == day)[0]
+        values_clim[iday, :] = ma.mean(dataset.values[t_index, :], axis=0)
+    for iday, day in enumerate(days_sorted):
+        t_index = np.where(days == day)[0]
+        dataset.values[t_index, :] = dataset.values[
+            t_index, :] - values_clim[iday, :]
+    return dataset
+
+
 def _rcmes_spatial_regrid(spatial_values, lat, lon, lat2, lon2, order=1):
     '''
     Spatial regrid from one set of lat,lon values onto a new set (lat2,lon2)
@@ -860,11 +921,9 @@ def _rcmes_spatial_regrid(spatial_values, lat, lon, lat2, lon2, order=1):
 
     nlat = spatial_values.shape[0]
     nlon = spatial_values.shape[1]
-    # print nlat, nlon, "lats, lons - incoming dataset"
 
     nlat2 = lat2.shape[0]
     nlon2 = lon2.shape[1]
-    # print nlat2, nlon2, "NEW lats, lons - for the new grid output"
 
     # To make our lives easier down the road, let's
     # turn these into arrays of x & y coords
@@ -875,13 +934,17 @@ def _rcmes_spatial_regrid(spatial_values, lat, lon, lat2, lon2, order=1):
     lati = lati.copy()
 
     # Now, we'll set points outside the boundaries to lie along an edge
-    loni[loni > lon.max()] = lon.max()
-    loni[loni < lon.min()] = lon.min()
+    loni_max_indices = np.where(loni > lon.max())
+    loni_min_indices = np.where(loni < lon.min())
+    loni[loni_max_indices] = lon.max()
+    loni[loni_min_indices] = lon.min()
 
     # To deal with the "hard" break, we'll have to treat y differently,
     # so we're just setting the min here...
-    lati[lati > lat.max()] = lat.max()
-    lati[lati < lat.min()] = lat.min()
+    lati_max_indices = np.where(lati > lat.max())
+    lati_min_indices = np.where(lati < lat.min())
+    lati[lati_max_indices] = lat.max()
+    lati[lati_min_indices] = lat.min()
 
     # We need to convert these to (float) indicies
     #   (xi should range from 0 to (nx - 1), etc)
@@ -915,7 +978,8 @@ def _rcmes_spatial_regrid(spatial_values, lat, lon, lat2, lon2, order=1):
         for axis in (0, 1):
             q_shifted = np.roll(spatial_values, shift=shift, axis=axis)
             idx = ~q_shifted.mask * spatial_values.mask
-            spatial_values.data[idx] = q_shifted[idx]
+            indices = np.where(idx)
+            spatial_values.data[indices] = q_shifted[indices]
 
     # Now we actually interpolate
     # map_coordinates does cubic interpolation by default,
@@ -937,8 +1001,10 @@ def _rcmes_spatial_regrid(spatial_values, lat, lon, lat2, lon2, order=1):
     # Make second map using nearest neighbour interpolation -use this to
     # determine locations with MDI and mask these
     qmdi = np.zeros_like(spatial_values)
-    qmdi[spatial_values.mask == True] = 1.
-    qmdi[spatial_values.mask == False] = 0.
+    spatial_indices_true = np.where(spatial_values.mask)
+    spatial_indices_false = np.where(not spatial_values.mask)
+    qmdi[spatial_indices_true] = 1.
+    qmdi[spatial_indices_false] = 0.
     qmdi_r = map_coordinates(qmdi, [lati, loni], order=order)
     qmdi_r = qmdi_r.reshape([nlat2, nlon2])
     mdimask = (qmdi_r != 0.0)
@@ -1000,19 +1066,20 @@ def _rcmes_calc_average_on_new_time_unit(data, dates, unit):
     :returns: meanstorem, newTimesList
     :rtype: 3D numpy masked array the same shape as the input array,
             list of python datetime objects
+    :raises: ValueError
     """
 
     # Check if the user-selected temporal grid is valid. If not, EXIT
     acceptable = ((unit == 'full') | (unit == 'annual') |
                   (unit == 'monthly') | (unit == 'daily'))
     if not acceptable:
-        print 'Error: unknown unit type selected for time averaging: EXIT'
-        return - 1, - 1, - 1, - 1
+        raise ValueError('Error: unknown unit type selected '
+                         'for time averaging: EXIT')
 
     nt, ny, nx = data.shape
     if unit == 'full':
         new_data = ma.mean(data, axis=0)
-        new_date = [dates[dates.size / 2]]
+        new_date = [dates[dates.size // 2]]
     if unit == 'annual':
         years = [d.year for d in dates]
         years_sorted = np.unique(years)
@@ -1051,8 +1118,8 @@ def _rcmes_calc_average_on_new_time_unit(data, dates, unit):
         for day in days_sorted:
             index = np.where(days == day)[0]
             new_data[it, :] = ma.mean(data[index, :], axis=0)
-            y = int(day / 10000)
-            m = int(day % 10000) / 100
+            y = int(day // 10000)
+            m = int(day % 10000) // 100
             d = int(day % 100)
             new_date.append(datetime.datetime(year=y, month=m, day=d))
             it = it + 1
@@ -1074,14 +1141,15 @@ def _rcmes_calc_average_on_new_time_unit_K(data, dates, unit):
     :returns: meanstorem, newTimesList
     :rtype: 3D numpy masked array the same shape as the input array,
             list of python datetime objects
+    :raises: ValueError
     """
 
     # Check if the user-selected temporal grid is valid. If not, EXIT
     acceptable = ((unit == 'full') | (unit == 'annual') |
                   (unit == 'monthly') | (unit == 'daily'))
     if not acceptable:
-        print 'Error: unknown unit type selected for time averaging: EXIT'
-        return - 1, - 1, - 1, - 1
+        raise ValueError('Error: unknown unit type selected '
+                         'for time averaging: EXIT')
 
     # Calculate arrays of: annual timeseries: year (2007,2007),
     #                      monthly time series: year-month (200701,200702),
@@ -1252,9 +1320,9 @@ def _congrid(a, newdims, method='linear', centre=False, minusone=False):
     old = np.array(a.shape)
     ndims = len(a.shape)
     if len(newdims) != ndims:
-        print "[congrid] dimensions error. " \
-              "This routine currently only supports " \
-              "rebinning to the same number of dimensions."
+        print("[congrid] dimensions error. "
+              "This routine currently only supports "
+              "rebinning to the same number of dimensions.")
         return None
     newdims = np.asarray(newdims, dtype=float)
     dimlist = []
@@ -1307,9 +1375,9 @@ def _congrid(a, newdims, method='linear', centre=False, minusone=False):
         newa = scipy.ndimage.map_coordinates(a, newcoords)
         return newa
     else:
-        print "Congrid error: Unrecognized interpolation type.\n", \
-              "Currently only \'neighbour\', \'nearest\',\'linear\',", \
-              "and \'spline\' are supported."
+        print("Congrid error: Unrecognized interpolation type.\n",
+              "Currently only \'neighbour\', \'nearest\',\'linear\',",
+              "and \'spline\' are supported.")
         return None
 
 
@@ -1359,7 +1427,7 @@ def _congrid_neighbor(values, new_dims, minus_one, offset):
     return new_values
 
 
-def _are_bounds_contained_by_dataset(bounds, dataset):
+def _are_bounds_contained_by_dataset(dataset, bounds):
     '''Check if a Dataset fully contains a bounds.
 
     :param bounds: The Bounds object to check.
@@ -1372,44 +1440,32 @@ def _are_bounds_contained_by_dataset(bounds, dataset):
         a ValueError otherwise
     '''
     lat_min, lat_max, lon_min, lon_max = dataset.spatial_boundaries()
-    start, end = dataset.time_range()
+    start, end = dataset.temporal_boundaries()
+    
     errors = []
 
     # TODO:  THIS IS TERRIBLY inefficent and we need to use a geometry
     # lib instead in the future
-    if not (np.round(lat_min, 3) <= np.round(bounds.lat_min, 3) <=
-            np.round(lat_max, 3)):
-        error = ("bounds.lat_min: %s is not between lat_min: %s and"
-                 " lat_max: %s" % (bounds.lat_min, lat_min, lat_max))
-        errors.append(error)
-
-    if not (np.round(lat_min, 3) <= np.round(bounds.lat_max, 3) <=
-            np.round(lat_max, 3)):
+    if (lat_min > bounds.lat_max):
         error = ("bounds.lat_max: %s is not between lat_min: %s and"
-                 "lat_max: %s" % (bounds.lat_max, lat_min, lat_max))
+                 " lat_max: %s" % (bounds.lat_max, lat_min, lat_max))
         errors.append(error)
 
-    if not (np.round(lon_min, 3) <= np.round(bounds.lon_min, 3) <=
-            np.round(lon_max, 3)):
-        error = ("bounds.lon_min: %s is not between lon_min: %s and"
-                 "lon_max: %s" % (bounds.lon_min, lon_min, lon_max))
-        errors.append(error)
-
-    if not (np.round(lon_min, 3) <= np.round(bounds.lon_max, 3) <=
-            np.round(lon_max, 3)):
+    if (lon_min > bounds.lon_max):
         error = ("bounds.lon_max: %s is not between lon_min: %s and"
-                 "lon_max: %s" % (bounds.lon_max, lon_min, lon_max))
+                 " lon_max: %s" % (bounds.lon_max, lon_min, lon_max))
         errors.append(error)
+    if not (bounds.start is None):
+        if not start <= bounds.start <= end:
+            error = ("bounds.start: %s is not between start: %s and end: %s" %
+                     (bounds.start, start, end))
+            errors.append(error)
 
-    if not start <= bounds.start <= end:
-        error = ("bounds.start: %s is not between start: %s and end: %s" %
-                 (bounds.start, start, end))
-        errors.append(error)
-
-    if not start <= bounds.end <= end:
-        error = ("bounds.end: %s is not between start: %s and end: %s" %
-                 (bounds.end, start, end))
-        errors.append(error)
+    if not (bounds.end is None):
+        if not start <= bounds.end <= end:
+            error = ("bounds.end: %s is not between start: %s and end: %s" %
+                     (bounds.end, start, end))
+            errors.append(error)
 
     if len(errors) == 0:
         return True
@@ -1418,7 +1474,7 @@ def _are_bounds_contained_by_dataset(bounds, dataset):
         raise ValueError(error_message)
 
 
-def _get_subregion_slice_indices(subregion, target_dataset):
+def _get_subregion_slice_indices(target_dataset, subregion):
     '''Get the indices for slicing Dataset values to generate the subregion.
 
     :param subregion: The Bounds that specify the subset of the Dataset
@@ -1435,8 +1491,12 @@ def _get_subregion_slice_indices(subregion, target_dataset):
     lonStart = min(np.nonzero(target_dataset.lons >= subregion.lon_min)[0])
     lonEnd = max(np.nonzero(target_dataset.lons <= subregion.lon_max)[0])
 
-    timeStart = min(np.nonzero(target_dataset.times >= subregion.start)[0])
-    timeEnd = max(np.nonzero(target_dataset.times <= subregion.end)[0])
+    if not (subregion.start is None):
+        timeStart = min(np.nonzero(target_dataset.times >= subregion.start)[0])
+        timeEnd = max(np.nonzero(target_dataset.times <= subregion.end)[0])
+    else:
+        timeStart = 0
+        timeEnd = len(target_dataset.times) -1
 
     return {
         "lat_start": latStart,
