@@ -16,30 +16,31 @@
 # under the License.
 
 '''
-Classes: 
+Classes:
     Evaluation - Container for running an evaluation
 '''
 
 import logging
-from metrics import Metric, UnaryMetric, BinaryMetric
-from dataset import Dataset, Bounds
+from ocw.metrics import Metric, UnaryMetric, BinaryMetric
+from ocw.dataset import Dataset, Bounds
 import ocw.dataset_processor as DSP
 
 import numpy.ma as ma
 
 logger = logging.getLogger(__name__)
 
+
 class Evaluation(object):
     '''Container for running an evaluation
 
-    An *Evaluation* is the running of one or more metrics on one or more 
+    An *Evaluation* is the running of one or more metrics on one or more
     target datasets and a (possibly optional) reference dataset. Evaluation
     can handle two types of metrics, ``unary`` and ``binary``. The validity
     of an Evaluation is dependent upon the number and type of metrics as well
     as the number of datasets.
 
     A ``unary`` metric is a metric that runs over a single dataset. If you add
-    a ``unary`` metric to the Evaluation you are only required to add a 
+    a ``unary`` metric to the Evaluation you are only required to add a
     reference dataset or a target dataset. If there are multiple datasets
     in the evaluation then the ``unary`` metric is run over all of them.
 
@@ -49,7 +50,7 @@ class Evaluation(object):
     are run over every (reference dataset, target dataset) pair in the
     Evaluation.
 
-    An Evaluation must have at least one metric to be valid. 
+    An Evaluation must have at least one metric to be valid.
     '''
 
     def __init__(self, reference, targets, metrics, subregions=None):
@@ -58,11 +59,11 @@ class Evaluation(object):
         :param reference: The reference Dataset for the evaluation.
         :type reference: :class:`dataset.Dataset`
 
-        :param targets: A list of one or more target datasets for the 
+        :param targets: A list of one or more target datasets for the
                 evaluation.
         :type targets: :class:`list` of :class:`dataset.Dataset`
 
-        :param metrics: A list of one or more Metric instances to run 
+        :param metrics: A list of one or more Metric instances to run
                 in the evaluation.
         :type metrics: :class:`list` of :mod:`metrics`
 
@@ -70,16 +71,16 @@ class Evaluation(object):
                 evaluation. A subregion is specified with a Bounds object.
         :type subregions: :class:`list` of :class:`dataset.Bounds`
 
-        :raises: ValueError 
+        :raises: ValueError
         '''
         #: The reference dataset.
         self._ref_dataset = reference
-        #: The target dataset(s) which should each be compared with 
+        #: The target dataset(s) which should each be compared with
         #: the reference dataset when the evaluation is run.
         self.target_datasets = []
         self.add_datasets(targets)
 
-        #: The list of "binary" metrics (A metric which takes two Datasets) 
+        #: The list of "binary" metrics (A metric which takes two Datasets)
         #: that the Evaluation should use.
         self.metrics = []
         #: The list of "unary" metrics (A metric which takes one Dataset) that
@@ -91,17 +92,17 @@ class Evaluation(object):
         self.add_metrics(metrics)
 
         #: An optional list of subregion bounds to use when running the
-        #: evaluation. 
+        #: evaluation.
         self._subregions = subregions
 
-        #: A list containing the results of running regular metric evaluations. 
+        #: A list containing the results of running regular metric evaluations.
         #: The shape of results is ``(num_target_datasets, num_metrics)`` if
         #: the user doesn't specify subregion information. Otherwise the shape
         #: is ``(num_target_datasets, num_metrics, num_subregions)``.
         self.results = []
-        #: A list containing the results of running the unary metric 
-        #: evaluations. The shape of unary_results is 
-        #: ``(num_targets, num_metrics)`` where ``num_targets = 
+        #: A list containing the results of running the unary metric
+        #: evaluations. The shape of unary_results is
+        #: ``(num_targets, num_metrics)`` where ``num_targets =
         #: num_target_ds + (1 if ref_dataset != None else 0``
         self.unary_results = []
 
@@ -139,7 +140,7 @@ class Evaluation(object):
     def add_dataset(self, target_dataset):
         '''Add a Dataset to the Evaluation.
 
-        A target Dataset is compared against the reference dataset when the 
+        A target Dataset is compared against the reference dataset when the
         Evaluation is run with one or more metrics.
 
         :param target_dataset: The target Dataset to add to the Evaluation.
@@ -160,7 +161,7 @@ class Evaluation(object):
     def add_datasets(self, target_datasets):
         '''Add multiple Datasets to the Evaluation.
 
-        :param target_datasets: The list of datasets that should be added to 
+        :param target_datasets: The list of datasets that should be added to
             the Evaluation.
         :type target_datasets: :class:`list` of :class:`dataset.Dataset`
 
@@ -207,7 +208,6 @@ class Evaluation(object):
         for metric in metrics:
             self.add_metric(metric)
 
-
     def run(self):
         '''Run the evaluation.
 
@@ -217,8 +217,8 @@ class Evaluation(object):
         target dataset.
 
         If there is subregion information provided then each dataset is subset
-        before being run through the binary metrics. 
-        
+        before being run through the binary metrics.
+
         ..note:: Only the binary metrics are subset with subregion information.
 
         Next, if there are any "unary" metrics they are run. Unary metrics are
@@ -243,7 +243,7 @@ class Evaluation(object):
 
     def _evaluation_is_valid(self):
         '''Check if the evaluation is well-formed.
-        
+
         * If there are no metrics or no datasets it's invalid.
         * If there is a unary metric there must be a reference dataset or at
             least one target dataset.
@@ -272,11 +272,11 @@ class Evaluation(object):
 
     def _run_subregion_evaluation(self):
         results = []
-        new_refs = [DSP.subset(s, self.ref_dataset) for s in self.subregions]
+        new_refs = [DSP.subset(self.ref_dataset, s) for s in self.subregions]
 
         for target in self.target_datasets:
             results.append([])
-            new_targets = [DSP.subset(s, target) for s in self.subregions]
+            new_targets = [DSP.subset(target, s) for s in self.subregions]
 
             for metric in self.metrics:
                 results[-1].append([])
@@ -313,10 +313,11 @@ class Evaluation(object):
     def _run_subregion_unary_evaluation(self):
         unary_results = []
         if self.ref_dataset:
-            new_refs = [DSP.subset(s, self.ref_dataset) for s in self.subregions]
+            new_refs = [DSP.subset(self.ref_dataset, s)
+                        for s in self.subregions]
 
         new_targets = [
-            [DSP.subset(s, t) for s in self.subregions]
+            [DSP.subset(t, s) for s in self.subregions]
             for t in self.target_datasets
         ]
 
@@ -332,7 +333,7 @@ class Evaluation(object):
                 for t in range(len(self.target_datasets)):
                     unary_results[-1][-1].append(metric.run(new_targets[t][i]))
 
-        return convert_unary_evaluation_result(unary_results, subregion = True)
+        return convert_unary_evaluation_result(unary_results, subregion=True)
 
     def __str__(self):
         formatted_repr = (
@@ -351,18 +352,19 @@ class Evaluation(object):
             str(self.subregions)
         )
 
-def convert_evaluation_result(evaluation_result, subregion = False):
+
+def convert_evaluation_result(evaluation_result, subregion=False):
     if not subregion:
         nmodel = len(evaluation_result)
         nmetric = len(evaluation_result[0])
-        results = [] 
+        results = []
         for imetric in range(nmetric):
-            if evaluation_result[0][imetric].ndim !=0:
+            if evaluation_result[0][imetric].ndim != 0:
                 result_shape = list(evaluation_result[0][imetric].shape)
                 result_shape.insert(0, nmodel)
                 result = ma.zeros(result_shape)
                 for imodel in range(nmodel):
-                    result[imodel,:] = evaluation_result[imodel][imetric]
+                    result[imodel, :] = evaluation_result[imodel][imetric]
             else:
                 result = ma.zeros(nmodel)
                 for imodel in range(nmodel):
@@ -378,32 +380,36 @@ def convert_evaluation_result(evaluation_result, subregion = False):
         for isubregion in range(nsubregion):
             subregion_results = []
             for imetric in range(nmetric):
-                if evaluation_result[0][imetric][isubregion].ndim !=0:
-                    result_shape = list(evaluation_result[0][imetric][isubregion].shape)
+                if evaluation_result[0][imetric][isubregion].ndim != 0:
+                    result_shape = list(evaluation_result[0][
+                                        imetric][isubregion].shape)
                     result_shape.insert(0, nmodel)
                     result = ma.zeros(result_shape)
                     for imodel in range(nmodel):
-                        result[imodel,:] = evaluation_result[imodel][imetric][isubregion]
+                        result[imodel, :] = evaluation_result[
+                            imodel][imetric][isubregion]
                 else:
                     result = ma.zeros(nmodel)
                     for imodel in range(nmodel):
-                        result[imodel] = evaluation_result[imodel][imetric][isubregion]
+                        result[imodel] = evaluation_result[
+                            imodel][imetric][isubregion]
                 subregion_results.append(result)
             results.append(subregion_results)
         return results
-             
-def convert_unary_evaluation_result(evaluation_result, subregion = False):
+
+
+def convert_unary_evaluation_result(evaluation_result, subregion=False):
     if not subregion:
         nmetric = len(evaluation_result)
         nmodel = len(evaluation_result[0])
         results = []
         for imetric in range(nmetric):
-            if evaluation_result[imetric][0].ndim !=0:
+            if evaluation_result[imetric][0].ndim != 0:
                 result_shape = list(evaluation_result[imetric][0].shape)
                 result_shape.insert(0, nmodel)
                 result = ma.zeros(result_shape)
                 for imodel in range(nmodel):
-                    result[imodel,:] = evaluation_result[imetric][imodel]
+                    result[imodel, :] = evaluation_result[imetric][imodel]
             else:
                 result = ma.zeros(nmodel)
                 for imodel in range(nmodel):
@@ -419,17 +425,19 @@ def convert_unary_evaluation_result(evaluation_result, subregion = False):
         for isubregion in range(nsubregion):
             subregion_results = []
             for imetric in range(nmetric):
-                if evaluation_result[imetric][isubregion][0].ndim !=0:
-                    result_shape = list(evaluation_result[imetric][isubregion][0].shape)
+                if evaluation_result[imetric][isubregion][0].ndim != 0:
+                    result_shape = list(evaluation_result[imetric][
+                                        isubregion][0].shape)
                     result_shape.insert(0, nmodel)
                     result = ma.zeros(result_shape)
                     for imodel in range(nmodel):
-                        result[imodel,:] = evaluation_result[imetric][isubregion][imodel]
+                        result[imodel, :] = evaluation_result[
+                            imetric][isubregion][imodel]
                 else:
-                    result = ma.zeros(nmodel) 
+                    result = ma.zeros(nmodel)
                     for imodel in range(nmodel):
-                        result[imodel] = evaluation_result[imetric][isubregion][imodel]
+                        result[imodel] = evaluation_result[
+                            imetric][isubregion][imodel]
                 subregion_results.append(result)
             results.append(subregion_results)
         return results
-
