@@ -18,15 +18,22 @@
 from tempfile import TemporaryFile
 import matplotlib as mpl
 import matplotlib.pyplot as plt
+from matplotlib.colors import BoundaryNorm
+from matplotlib import rcParams, cm
+from matplotlib.patches import Polygon
+from matplotlib.collections import PatchCollection
 from mpl_toolkits.basemap import Basemap
 from mpl_toolkits.axes_grid1 import ImageGrid
 import scipy.stats.mstats as mstats
 import numpy as np
 import numpy.ma as ma
 
+import ocw.utils as utils
+
 # Set the default colormap to coolwarm
 mpl.rc('image', cmap='coolwarm')
 
+
 def set_cmap(name):
     '''
     Sets the default colormap (eg when setting cmap=None in a function)
@@ -43,6 +50,7 @@ def set_cmap(name):
     cmap = plt.get_cmap(name)
     mpl.rc('image', cmap=cmap.name)
 
+
 def _nice_intervals(data, nlevs):
     '''
     Purpose::
@@ -63,20 +71,25 @@ def _nice_intervals(data, nlevs):
     data = data.ravel()
     mn = mstats.scoreatpercentile(data, 5)
     mx = mstats.scoreatpercentile(data, 95)
-    #if there min less than 0 and
-    # or max more than 0 
-    #put 0 in center of color bar
+    # if min less than 0 and or max more than 0 put 0 in center of color bar
     if mn < 0 and mx > 0:
         level = max(abs(mn), abs(mx))
         mnlvl = -1 * level
         mxlvl = level
-    #if min is larger than 0 then
-    #have color bar between min and max
+    # if min is larger than 0 then have color bar between min and max
     else:
         mnlvl = mn
         mxlvl = mx
+    
+    # hack to make generated intervals from mpl the same for all versions
+    autolimit_mode = mpl.rcParams.get('axes.autolimit_mode')
+    if autolimit_mode:
+        mpl.rc('axes', autolimit_mode='round_numbers')
+    
     locator = mpl.ticker.MaxNLocator(nlevs)
     clevs = locator.tick_values(mnlvl, mxlvl)
+    if autolimit_mode:
+        mpl.rc('axes', autolimit_mode=autolimit_mode)
 
     # Make sure the bounds of clevs are reasonable since sometimes
     # MaxNLocator gives values outside the domain of the input data
@@ -102,7 +115,8 @@ def _best_grid_shape(nplots, oldshape):
     size = nrows * ncols
     diff = size - nplots
     if diff < 0:
-        raise ValueError('gridshape=(%d, %d): Cannot fit enough subplots for data' %(nrows, ncols))
+        raise ValueError(
+            'gridshape=(%d, %d): Cannot fit enough subplots for data' % (nrows, ncols))
     else:
         # If the user enters an excessively large number of
         # rows and columns for gridshape, automatically
@@ -120,6 +134,7 @@ def _best_grid_shape(nplots, oldshape):
         newshape = nrows, ncols
         return newshape
 
+
 def _fig_size(gridshape, aspect=None):
     '''
     Purpose::
@@ -142,15 +157,16 @@ def _fig_size(gridshape, aspect=None):
     nrows, ncols = gridshape
     if nrows >= ncols:
         # If more rows keep width constant
-        width, height = (aspect * 5.5), 5.5 * (nrows / ncols)
+        width, height = (aspect * 5.5), 5.5 * (nrows // ncols)
     else:
         # If more columns keep height constant
-        width, height = (aspect * 5.5) * (ncols / nrows), 5.5
+        width, height = (aspect * 5.5) * (ncols // nrows), 5.5
 
     return width, height
 
+
 def draw_taylor_diagram(results, names, refname, fname, fmt='png',
-                        gridshape=(1,1), ptitle='', subtitles=None,
+                        gridshape=(1, 1), ptitle='', subtitles=None,
                         pos='upper right', frameon=True, radmax=1.5):
     ''' Draw a Taylor diagram.
 
@@ -176,7 +192,7 @@ def draw_taylor_diagram(results, names, refname, fname, fmt='png',
 
     :param ptitle: (Optional) plot title.
     :type ptitle: :mod:`string`
-    
+
     :param subtitles: (Optional) list of strings specifying the title for each
         subplot.
     :type subtitles: :class:`list` of :mod:`string`
@@ -212,7 +228,8 @@ def draw_taylor_diagram(results, names, refname, fname, fmt='png',
         rect = str(rect[0]) + str(rect[1]) + str(rect[2])
 
         # Create Taylor Diagram object
-        dia = TaylorDiagram(1, fig=fig, rect=rect, label=refname, radmax=radmax)
+        dia = TaylorDiagram(1, fig=fig, rect=rect,
+                            label=refname, radmax=radmax)
         for i, (stddev, corrcoef) in enumerate(data):
             dia.add_sample(stddev, corrcoef, marker='$%d$' % (i + 1), ms=6,
                            label=names[i])
@@ -230,9 +247,10 @@ def draw_taylor_diagram(results, names, refname, fname, fmt='png',
     # Add title and save the figure
     fig.suptitle(ptitle)
     plt.tight_layout(.05, .05)
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
+
 def draw_subregions(subregions, lats, lons, fname, fmt='png', ptitle='',
                     parallels=None, meridians=None, subregion_masks=None):
     ''' Draw subregion domain(s) on a map.
@@ -301,9 +319,11 @@ def draw_subregions(subregions, lats, lons, fname, fmt='png', ptitle='',
         dlatlon = np.round(length, decimals=-1)
 
     if meridians is None:
-        meridians = np.r_[np.arange(0, -180, -dlatlon)[::-1], np.arange(0, 180, dlatlon)]
+        meridians = np.r_[
+            np.arange(0, -180, -dlatlon)[::-1], np.arange(0, 180, dlatlon)]
     if parallels is None:
-        parallels = np.r_[np.arange(0, -90, -dlatlon)[::-1], np.arange(0, 90, dlatlon)]
+        parallels = np.r_[np.arange(0, -90, -dlatlon)
+                          [::-1], np.arange(0, 90, dlatlon)]
 
     # Draw parallels / meridians
     m.drawmeridians(meridians, labels=[0, 0, 0, 1], linewidth=.75, fontsize=10)
@@ -336,15 +356,17 @@ def draw_subregions(subregions, lats, lons, fname, fmt='png', ptitle='',
 
         # Label the subregion
         xm, ym = x.mean(), y.mean()
-        m.plot(xm, ym, marker='$%s$' %("R"+str(i+1)), markersize=12, color='k')
+        m.plot(xm, ym, marker='$%s$' %
+               ("R" + str(i + 1)), markersize=12, color='k')
 
     # Add the title
     ax.set_title(ptitle)
 
     # Save the figure
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
+
 def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
                      xlabel='', ylabel='', ptitle='', subtitles=None,
                      label_month=False, yscale='linear', aspect=None):
@@ -372,7 +394,7 @@ def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
 
     :param xlabel: (Optional) x-axis title.
     :type xlabel: :mod:`string`
-    
+
     :param ylabel: (Optional) y-axis title.
     :type ylabel: :mod:`string`
 
@@ -381,7 +403,7 @@ def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
 
     :param subtitles: (Optional) list of titles for each subplot.
     :type subtitles: :class:`list` of :mod:`string`
-    
+
     :param label_month: (Optional) flag to toggle drawing month labels on the
         x-axis.
     :type label_month: :class:`bool`
@@ -389,7 +411,7 @@ def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
     :param yscale: (Optional) y-axis scale value, 'linear' for linear and 'log'
         for log base 10.
     :type yscale: :mod:`string`
-    
+
     :param aspect: (Optional) approximate aspect ratio of each subplot
         (width / height). Default is 8.5 / 5.5
     :type aspect: :class:`float`
@@ -458,7 +480,8 @@ def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
 
     # Create a master axes rectangle for figure wide labels
     fax = fig.add_subplot(111, frameon=False)
-    fax.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
+    fax.tick_params(labelcolor='none', top='off',
+                    bottom='off', left='off', right='off')
     fax.set_ylabel(ylabel)
     fax.set_title(ptitle, fontsize=16)
     fax.title.set_y(1.04)
@@ -470,7 +493,7 @@ def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
     cax.set_xticks([])
     cax.set_yticks([])
     cax.legend((lines), labels, loc='upper center', ncol=10, fontsize='small',
-                   mode='expand', frameon=False)
+               mode='expand', frameon=False)
 
     # Note that due to weird behavior by axes_grid, it is more convenient to
     # place the x-axis label relative to the colorbar axes instead of the
@@ -485,18 +508,19 @@ def draw_time_series(results, times, labels, fname, fmt='png', gridshape=(1, 1),
             xtick.set_rotation(30)
 
     # Save the figure
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
-def draw_barchart(results, yvalues, fname, ptitle='', fmt='png', 
-                     xlabel='', ylabel=''):
+
+def draw_barchart(results, yvalues, fname, ptitle='', fmt='png',
+                  xlabel='', ylabel=''):
     ''' Draw a barchart.
 
     :param results: 1D array of  data.
     :type results: :class:`numpy.ndarray`
 
     :param yvalues: List of y-axis labels
-    :type times: :class:`list` 
+    :type times: :class:`list`
 
     :param fname: Filename of the plot.
     :type fname: :mod:`string`
@@ -509,36 +533,38 @@ def draw_barchart(results, yvalues, fname, ptitle='', fmt='png',
 
     :param xlabel: (Optional) x-axis title.
     :type xlabel: :mod:`string`
-    
+
     :param ylabel: (Optional) y-axis title.
     :type ylabel: :mod:`string`
 
     '''
 
-    y_pos = list(range(len(yvalues))) 
-    fig = plt.figure() 
+    y_pos = list(range(len(yvalues)))
+    fig = plt.figure()
     fig.set_size_inches((11., 8.5))
     fig.dpi = 300
     ax = plt.subplot(111)
     plt.barh(y_pos, results, align="center", height=0.8, linewidth=0)
     plt.yticks(y_pos, yvalues)
-    plt.tick_params(axis="both", which="both", bottom="on", top="off",labelbottom="on", left="off", right="off", labelleft="on")
+    plt.tick_params(axis="both", which="both", bottom="on", top="off",
+                    labelbottom="on", left="off", right="off", labelleft="on")
     ax.spines["top"].set_visible(False)
     ax.spines["right"].set_visible(False)
 
-    ymin = min(y_pos) 
+    ymin = min(y_pos)
     ymax = max(y_pos)
-    ymin = min((ymin - ((ymax - ymin) * 0.1)/2),0.5) 
+    ymin = min((ymin - ((ymax - ymin) * 0.1) / 2), 0.5)
     ymax = ymax + ((ymax - ymin) * 0.1)
     ax.set_ylim((ymin, ymax))
     plt.xlabel(xlabel)
     plt.tight_layout()
-       
+
     # Save the figure
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
-def draw_marker_on_map(lat, lon, fname, fmt='png', location_name=' ',gridshape=(1,1)):
+
+def draw_marker_on_map(lat, lon, fname, fmt='png', location_name=' ', gridshape=(1, 1)):
     '''
     Purpose::
         Draw a marker on a map
@@ -547,31 +573,34 @@ def draw_marker_on_map(lat, lon, fname, fmt='png', location_name=' ',gridshape=(
         lat - latitude for plotting a marker
         lon - longitude for plotting a marker
         fname  - a string specifying the filename of the plot
-    '''   
+    '''
     fig = plt.figure()
     fig.dpi = 300
     ax = fig.add_subplot(111)
-    
-    m = Basemap(projection='cyl', resolution = 'c', llcrnrlat =lat-30, urcrnrlat = lat+30, llcrnrlon = lon-60, urcrnrlon = lon+60)
+
+    m = Basemap(projection='cyl', resolution='c', llcrnrlat=lat -
+                30, urcrnrlat=lat + 30, llcrnrlon=lon - 60, urcrnrlon=lon + 60)
     m.drawcoastlines(linewidth=1)
     m.drawcountries(linewidth=1)
     m.drawmapboundary(fill_color='aqua')
-    m.fillcontinents(color='coral',lake_color='aqua')
+    m.fillcontinents(color='coral', lake_color='aqua')
     m.ax = ax
-   
-    xpt,ypt = m(lon,lat)
-    m.plot(xpt,ypt,'bo')  # plot a blue dot there
+
+    xpt, ypt = m(lon, lat)
+    m.plot(xpt, ypt, 'bo')  # plot a blue dot there
     # put some text next to the dot, offset a little bit
     # (the offset is in map projection coordinates)
-    plt.text(xpt+0.5, ypt+1.5,location_name+'\n(lon: %5.1f, lat: %3.1f)' % (lon, lat)) 
-                       
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    plt.text(xpt + 0.5, ypt + 1.5, location_name +
+             '\n(lon: %5.1f, lat: %3.1f)' % (lon, lat))
+
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
+
 def draw_contour_map(dataset, lats, lons, fname, fmt='png', gridshape=(1, 1),
                      clabel='', ptitle='', subtitles=None, cmap=None,
                      clevs=None, nlevs=10, parallels=None, meridians=None,
-                     extend='neither', aspect=8.5/2.5):
+                     extend='neither', aspect=8.5 / 2.5):
     ''' Draw a multiple panel contour map plot.
 
     :param dataset: 3D array of data to be plotted with shape (nT, nLat, nLon).
@@ -609,7 +638,7 @@ def draw_contour_map(dataset, lats, lons, fname, fmt='png', gridshape=(1, 1),
 
     :param clevs: (Optional) contour levels values.
     :type clevs: :class:`list` of :class:`int` or :class:`float`
-    
+
     :param nlevs: (Optional) target number of contour levels if clevs is None.
     :type nlevs: :class:`int`
 
@@ -667,8 +696,8 @@ def draw_contour_map(dataset, lats, lons, fname, fmt='png', gridshape=(1, 1),
     lonmax = lons.max()
     latmin = lats.min()
     latmax = lats.max()
-    m = Basemap(projection = 'cyl', llcrnrlat = latmin, urcrnrlat = latmax,
-                llcrnrlon = lonmin, urcrnrlon = lonmax, resolution = 'l')
+    m = Basemap(projection='cyl', llcrnrlat=latmin, urcrnrlat=latmax,
+                llcrnrlon=lonmin, urcrnrlon=lonmax, resolution='l')
 
     # Convert lats and lons to projection coordinates
     if lats.ndim == 1 and lons.ndim == 1:
@@ -692,11 +721,13 @@ def draw_contour_map(dataset, lats, lons, fname, fmt='png', gridshape=(1, 1),
     elif length <= 5:
         dlatlon = 5
     else:
-        dlatlon = np.round(length, decimals = -1)
+        dlatlon = np.round(length, decimals=-1)
     if meridians is None:
-        meridians = np.r_[np.arange(0, -180, -dlatlon)[::-1], np.arange(0, 180, dlatlon)]
+        meridians = np.r_[
+            np.arange(0, -180, -dlatlon)[::-1], np.arange(0, 180, dlatlon)]
     if parallels is None:
-        parallels = np.r_[np.arange(0, -90, -dlatlon)[::-1], np.arange(0, 90, dlatlon)]
+        parallels = np.r_[np.arange(0, -90, -dlatlon)
+                          [::-1], np.arange(0, 90, dlatlon)]
 
     x, y = m(lons, lats)
     for i, ax in enumerate(grid):
@@ -720,7 +751,8 @@ def draw_contour_map(dataset, lats, lons, fname, fmt='png', gridshape=(1, 1),
             ax.set_title(subtitles[i], fontsize='small')
 
     # Add colorbar
-    cbar = fig.colorbar(cs, cax=ax.cax, drawedges=True, orientation='horizontal', extendfrac='auto')
+    cbar = fig.colorbar(cs, cax=ax.cax, drawedges=True,
+                        orientation='horizontal', extendfrac='auto')
     cbar.set_label(clabel)
     cbar.set_ticks(clevs)
     cbar.ax.tick_params(labelsize=6)
@@ -739,9 +771,10 @@ def draw_contour_map(dataset, lats, lons, fname, fmt='png', gridshape=(1, 1),
 
     # Add figure title
     fig.suptitle(ptitle, y=ymax + .06, fontsize=16)
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
+
 def draw_portrait_diagram(results, rowlabels, collabels, fname, fmt='png',
                           gridshape=(1, 1), xlabel='', ylabel='', clabel='',
                           ptitle='', subtitles=None, cmap=None, clevs=None,
@@ -761,7 +794,7 @@ def draw_portrait_diagram(results, rowlabels, collabels, fname, fmt='png',
 
     :param fname: Filename of the plot.
     :type fname: :mod:`string`
-    
+
     :param fmt: (Optional) filetype for the output.
     :type fmt: :mod:`string`
 
@@ -817,7 +850,8 @@ def draw_portrait_diagram(results, rowlabels, collabels, fname, fmt='png',
     # the input data too
     prows, pcols = results.shape[1:]
     if len(rowlabels) != prows or len(collabels) != pcols:
-        raise ValueError('rowlabels and collabels must have %d and %d elements respectively' %(prows, pcols))
+        raise ValueError(
+            'rowlabels and collabels must have %d and %d elements respectively' % (prows, pcols))
 
     # Set up the figure
     width, height = _fig_size(gridshape)
@@ -853,7 +887,8 @@ def draw_portrait_diagram(results, rowlabels, collabels, fname, fmt='png',
     # Do the plotting
     for i, ax in enumerate(grid):
         data = results[i]
-        cs = ax.matshow(data, cmap=cmap, aspect='auto', origin='lower', norm=norm)
+        cs = ax.matshow(data, cmap=cmap, aspect='auto',
+                        origin='lower', norm=norm)
 
         # Add grid lines
         ax.xaxis.set_ticks(np.arange(data.shape[1] + 1))
@@ -873,11 +908,12 @@ def draw_portrait_diagram(results, rowlabels, collabels, fname, fmt='png',
         # Add axes title
         if subtitles is not None:
             ax.text(0.5, 1.04, subtitles[i], va='center', ha='center',
-                    transform = ax.transAxes, fontsize='small')
+                    transform=ax.transAxes, fontsize='small')
 
     # Create a master axes rectangle for figure wide labels
     fax = fig.add_subplot(111, frameon=False)
-    fax.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
+    fax.tick_params(labelcolor='none', top='off',
+                    bottom='off', left='off', right='off')
     fax.set_ylabel(ylabel)
     fax.set_title(ptitle, fontsize=16)
     fax.title.set_y(1.04)
@@ -899,9 +935,10 @@ def draw_portrait_diagram(results, rowlabels, collabels, fname, fmt='png',
     cax.title.set_y(1.5)
 
     # Save the figure
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
     fig.clf()
 
+
 class TaylorDiagram(object):
     """ Taylor diagram helper class
 
@@ -929,18 +966,18 @@ class TaylorDiagram(object):
         tr = PolarAxes.PolarTransform()
 
         # Correlation labels
-        rlocs = np.concatenate((np.arange(10)/10.,[0.95,0.99]))
+        rlocs = np.concatenate((np.arange(10) / 10., [0.95, 0.99]))
         tlocs = np.arccos(rlocs)        # Conversion to polar angles
         gl1 = GF.FixedLocator(tlocs)    # Positions
-        tf1 = GF.DictFormatter(dict(zip(tlocs, map(str,rlocs))))
+        tf1 = GF.DictFormatter(dict(zip(tlocs, map(str, rlocs))))
 
         # Standard deviation axis extent
         self.smin = 0
-        self.smax = radmax*self.refstd
+        self.smax = radmax * self.refstd
 
         ghelper = FA.GridHelperCurveLinear(tr,
-                                           extremes=(0,np.pi/2, # 1st quadrant
-                                                     self.smin,self.smax),
+                                           extremes=(0, np.pi / 2,  # 1st quadrant
+                                                     self.smin, self.smax),
                                            grid_locator1=gl1,
                                            tick_formatter1=tf1,
                                            )
@@ -958,7 +995,7 @@ class TaylorDiagram(object):
         ax.axis["top"].label.set_axis_direction("top")
         ax.axis["top"].label.set_text("Correlation")
 
-        ax.axis["left"].set_axis_direction("bottom") # "X axis"
+        ax.axis["left"].set_axis_direction("bottom")  # "X axis"
         ax.axis["left"].label.set_text("Standard deviation")
 
         ax.axis["right"].set_axis_direction("top")   # "Y axis"
@@ -977,9 +1014,9 @@ class TaylorDiagram(object):
         # print "Reference std:", self.refstd
         l, = self.ax.plot([0], self.refstd, 'k*',
                           ls='', ms=10, label=label)
-        t = np.linspace(0, np.pi/2)
+        t = np.linspace(0, np.pi / 2)
         r = np.zeros_like(t) + self.refstd
-        self.ax.plot(t,r, 'k--', label='_')
+        self.ax.plot(t, r, 'k--', label='_')
 
         # Collect sample points for latter use (e.g. legend)
         self.samplePoints = [l]
@@ -990,7 +1027,7 @@ class TaylorDiagram(object):
         command."""
 
         l, = self.ax.plot(np.arccos(corrcoef), stddev,
-                          *args, **kwargs) # (theta,radius)
+                          *args, **kwargs)  # (theta,radius)
         self.samplePoints.append(l)
 
         return l
@@ -998,10 +1035,11 @@ class TaylorDiagram(object):
     def add_rms_contours(self, levels=5, **kwargs):
         """Add constant centered RMS difference contours."""
 
-        rs,ts = np.meshgrid(np.linspace(self.smin,self.smax),
-                            np.linspace(0,np.pi/2))
+        rs, ts = np.meshgrid(np.linspace(self.smin, self.smax),
+                             np.linspace(0, np.pi / 2))
         # Compute centered RMS difference
-        rms = np.sqrt(self.refstd**2 + rs**2 - 2*self.refstd*rs*np.cos(ts))
+        rms = np.sqrt(self.refstd**2 + rs**2 - 2 *
+                      self.refstd * rs * np.cos(ts))
 
         contours = self.ax.contour(ts, rs, rms, levels, **kwargs)
 
@@ -1011,16 +1049,17 @@ class TaylorDiagram(object):
 
         t = np.linspace(np.arccos(corr1), np.arccos(corr2))
         r = np.zeros_like(t) + std
-        return self.ax.plot(t,r,'red', linewidth=2)
+        return self.ax.plot(t, r, 'red', linewidth=2)
 
-    def add_contours(self,std1,corr1,std2,corr2, **kwargs):
+    def add_contours(self, std1, corr1, std2, corr2, **kwargs):
         """Add a line between two points
         [std1, corr1] and [std2, corr2]"""
 
         t = np.linspace(np.arccos(corr1), np.arccos(corr2))
         r = np.linspace(std1, std2)
 
-        return self.ax.plot(t,r,'red',linewidth=2)
+        return self.ax.plot(t, r, 'red', linewidth=2)
+
 
 def draw_histogram(dataset_array, data_names, fname, fmt='png', nbins=10):
     '''
@@ -1032,25 +1071,206 @@ def draw_histogram(dataset_array, data_names, fname, fmt='png', nbins=10):
         data_names    - a list of data names  ['name1','name2',....]
         fname  - a string specifying the filename of the plot
         bins - number of bins
-    '''    
+    '''
     fig = plt.figure()
     fig.dpi = 300
     ndata = len(dataset_array)
-   
+
     data_min = 500.
     data_max = 0.
- 
-    for data in dataset_array:
-        data_min = np.min([data_min,data.min()]) 
-        data_max = np.max([data_max,data.max()]) 
 
-    bins = np.linspace(np.round(data_min), np.round(data_max+1), nbins)
-    for idata,data in enumerate(dataset_array):
-        ax = fig.add_subplot(ndata, 1, idata+1)
-        ax.hist(data, bins, alpha = 0.5, label=data_names[idata], normed = True)
+    for data in dataset_array:
+        data_min = np.min([data_min, data.min()])
+        data_max = np.max([data_max, data.max()])
+
+    bins = np.linspace(np.round(data_min), np.round(data_max + 1), nbins)
+    for idata, data in enumerate(dataset_array):
+        ax = fig.add_subplot(ndata, 1, idata + 1)
+        ax.hist(data, bins, alpha=0.5, label=data_names[idata], normed=True)
         leg = ax.legend()
         leg.get_frame().set_alpha(0.5)
-        ax.set_xlim([data_min-(data_max-data_min)*0.15, data_max+(data_max-data_min)*0.15])
-        
-    fig.savefig('%s.%s' %(fname, fmt), bbox_inches='tight', dpi=fig.dpi)
- 
+        ax.set_xlim([data_min - (data_max - data_min) * 0.15,
+                     data_max + (data_max - data_min) * 0.15])
+
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+
+def fill_US_states_with_color(regions, fname, fmt='png', ptitle='',
+                              colors=False, values=None, region_names=None):
+    
+    ''' Fill the States over the contiguous US with colors 
+   
+    :param regions: The list of subregions(lists of US States) to be filled 
+                    with different colors.
+    :type regions: :class:`list`  
+    :param fname: The filename of the plot.
+    :type fname: :mod:`string`
+    :param fmt: (Optional) filetype for the output.
+    :type fmt: :mod:`string`
+    :param ptitle: (Optional) plot title.
+    :type ptitle: :mod:`string`
+    :param colors: (Optional) : If True, each region will be filled  
+                                with different colors without using values
+    :type colors: :class:`bool` 
+    :param values: (Optional) : If colors==False, color for each region scales 
+                                an associated element in values   
+    :type values: :class:`numpy.ndarray` 
+    '''
+
+    nregion = len(regions)
+    if colors:
+        cmap = plt.cm.rainbow
+    if not (values is None):
+        cmap = plt.cm.seismic
+        max_abs = np.abs(values).max()
+
+    # Set up the figure
+    fig = plt.figure()
+    fig.set_size_inches((8.5, 11.))
+    fig.dpi = 300
+    ax = fig.add_subplot(111)
+
+    # create the map
+    m = Basemap(llcrnrlon=-127,llcrnrlat=22,urcrnrlon=-65,urcrnrlat=52,
+                ax=ax)
+
+    for iregion, region in enumerate(regions):
+        shapes = utils.shapefile_boundary('us_states', region)
+        patches=[]
+        lats=np.empty(0)
+        lons=np.empty(0)
+        for shape in shapes:
+            patches.append(Polygon(np.array(shape), True))
+            
+            lons = np.append(lons, shape[:,0])
+            lats = np.append(lats, shape[:,1])
+        if colors:
+            color_to_fill=cmap((iregion+0.5)/nregion)
+        if not (values is None):
+            value = values[iregion]
+            color_to_fill = cmap(0.5+np.sign(value)*abs(value)/max_abs*0.45)
+        ax.add_collection(PatchCollection(patches, facecolor=color_to_fill))
+        if region_names:
+            ax.text(lons.mean(), lats.mean(), region_names[iregion],
+                    ha='center', va='center', fontsize=10)
+    m.drawcountries(linewidth=0.)
+
+    # Add the title
+    ax.set_title(ptitle)
+    # Save the figure
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight', dpi=fig.dpi)
+    fig.clf()
+
+def draw_plot_to_compare_trends(obs_data, ens_data, model_data,
+                          fname, fmt='png', ptitle='', data_labels=None,
+                          xlabel='', ylabel=''):
+
+    ''' Fill the States over the contiguous US with colors 
+   
+    :param obs_data: An array of observed trend and standard errors for regions
+    :type obs_data: :class:'numpy.ndarray'
+    :param ens_data: An array of trend and standard errors from a multi-model ensemble for regions 
+    :type ens_data: : class:'numpy.ndarray'
+    :param model_data: An array of trends from models for regions 
+    :type model_data: : class:'numpy.ndarray'
+    :param fname: The filename of the plot.
+    :type fname: :mod:`string`
+    :param fmt: (Optional) filetype for the output.
+    :type fmt: :mod:`string`
+    :param ptitle: (Optional) plot title.
+    :type ptitle: :mod:`string`
+    :param data_labels: (Optional) names of the regions
+    :type data_labels: :mod:`list`
+    :param xlabel: (Optional) a label for x-axis
+    :type xlabel: :mod:`string`
+    :param ylabel: (Optional) a label for y-axis  
+    :type ylabel: :mod:`string`
+    '''
+    nregions = obs_data.shape[1] 
+
+    # Set up the figure
+    fig = plt.figure()
+    fig.set_size_inches((8.5, 11.))
+    fig.dpi = 300
+    ax = fig.add_subplot(111)
+  
+    b_plot = ax.boxplot(model_data, widths=np.repeat(0.2, nregions), positions=np.arange(nregions)+1.3) 
+    plt.setp(b_plot['medians'], color='black')
+    plt.setp(b_plot['whiskers'], color='black')
+    plt.setp(b_plot['boxes'], color='black')
+    plt.setp(b_plot['fliers'], color='black')
+    ax.errorbar(np.arange(nregions)+0.8, obs_data[0,:], yerr=obs_data[1,:], 
+                fmt='o', color='r', ecolor='r')    
+    ax.errorbar(np.arange(nregions)+1., ens_data[0,:], yerr=ens_data[1,:], 
+                fmt='o', color='b', ecolor='b')    
+    ax.set_xticks(np.arange(nregions)+1)
+    ax.set_xlim([0, nregions+1])
+    
+    if data_labels:
+        ax.set_xticklabels(data_labels)
+    fig.savefig('%s.%s' % (fname, fmt), bbox_inches='tight') 
+
+def draw_precipitation_JPDF (plot_data, plot_level, x_ticks, x_names,y_ticks,y_names, 
+               output_file, title=None, diff_plot=False, cmap = cm.BrBG, 
+               cbar_ticks=[0.01, 0.10, 0.5, 2, 5, 25], 
+               cbar_label=['0.01', '0.10', '0.5', '2', '5', '25']):
+    '''
+    :param plot_data: a numpy array of data to plot (dimY, dimX)
+    :type plot_data: :class:'numpy.ndarray'
+    :param plot_level: levels to plot
+    :type plot_level: :class:'numpy.ndarray'
+    :param x_ticks: x values where tick makrs are located
+    :type x_ticks: :class:'numpy.ndarray'                 
+    :param x_names: labels for the ticks on x-axis (dimX)
+    :type x_names: :class:'list'
+    :param y_ticks: y values where tick makrs are located
+    :type y_ticks: :class:'numpy.ndarray'                 
+    :param y_names: labels for the ticks on y-axis (dimY)
+    :type y_names: :class:'list'
+    :param output_file: name of output png file
+    :type output_file: :mod:'string'
+    :param title: (Optional) title of the plot
+    :type title: :mod:'string'
+    :param diff_plot: (Optional) if true, a difference plot will be generated
+    :type diff_plot: :mod:'bool'
+    :param cbar_ticks: (Optional) tick marks for the color bar
+    :type cbar_ticks: :class:'list'
+    :param cbar_label: (Optional) lables for the tick marks of the color bar
+    :type cbar_label: :class:'list'
+    '''
+
+    if diff_plot:
+        cmap = cm.RdBu_r
+
+    fig = plt.figure()
+    sb = fig.add_subplot(111)
+    dimY, dimX = plot_data.shape
+    plot_data2 = np.zeros([dimY,dimX]) # sectioned array for plotting
+
+    # fontsize 
+    rcParams['axes.labelsize'] = 8
+    rcParams['xtick.labelsize'] = 8
+    rcParams['ytick.labelsize'] = 8
+    # assign the values in plot_level to plot_data
+    for iy in range(dimY):
+         for ix in range(dimX):
+             if plot_data[iy,ix] <= np.min(plot_level):
+                 plot_data2[iy,ix] = -1.
+             else:
+                 plot_data2[iy,ix] = plot_level[np.where(plot_level <= plot_data[iy,ix])].max()
+    sb.set_xticks(x_ticks)
+    sb.set_xticklabels(x_names)
+    sb.set_yticks(y_ticks)
+    sb.set_yticklabels(y_names)
+
+    norm = BoundaryNorm(plot_level[1:], cmap.N)
+    a=sb.pcolor(plot_data2, edgecolors = 'k', linewidths=0.5, cmap = cmap, norm = norm)
+    a.cmap.set_under('w')
+    sb.set_xlabel('Spell duration [hrs]')
+    sb.set_ylabel('Peak rainfall [mm/hr]')
+    cax = fig.add_axes([0.2, -0.06, 0.6, 0.02])
+    cbar = plt.colorbar(a, cax=cax, orientation = 'horizontal', extend='both')
+    cbar.set_ticks(cbar_ticks)
+    cbar.set_ticklabels(cbar_label)
+    if title:
+        fig.suptitle(title)
+    fig.savefig(output_file, dpi=600,bbox_inches='tight')
