@@ -30,6 +30,7 @@ logging.basicConfig(level=logging.CRITICAL)
 
 
 class TestTemporalSubset(unittest.TestCase):
+
     def setUp(self):
         self.ten_year_dataset = ten_year_monthly_dataset()
 
@@ -37,22 +38,22 @@ class TestTemporalSubset(unittest.TestCase):
         self.dataset_times = np.array([datetime.datetime(year, month, 1)
                                        for year in range(2000, 2010)
                                        for month in range(1, 6)])
-        self.tempSubset = dp.temporal_subset(1, 5, self.ten_year_dataset)
+        self.tempSubset = dp.temporal_subset(self.ten_year_dataset, 1, 5)
         np.testing.assert_array_equal(
             self.dataset_times, self.tempSubset.times)
 
     def test_temporal_subset_with_average_time(self):
         self.dataset_times = np.array([datetime.datetime(year, 2, 1)
                                        for year in range(2000, 2010)])
-        self.tempSubset = dp.temporal_subset(1, 3,
-                                             self.ten_year_dataset,
+        self.tempSubset = dp.temporal_subset(self.ten_year_dataset,
+                                             1, 3,
                                              average_each_year=True)
         np.testing.assert_array_equal(self.dataset_times,
                                       self.tempSubset.times)
 
     def test_temporal_subset_with_average_values(self):
-        self.tempSubset = dp.temporal_subset(1, 3,
-                                             self.ten_year_dataset,
+        self.tempSubset = dp.temporal_subset(self.ten_year_dataset,
+                                             1, 3,
                                              average_each_year=True)
         self.dataset_values = np.ones([len(self.tempSubset.times),
                                        len(self.ten_year_dataset.lats),
@@ -61,8 +62,8 @@ class TestTemporalSubset(unittest.TestCase):
                                       self.tempSubset.values)
 
     def test_temporal_subset_attributes(self):
-        self.tempSubset = dp.temporal_subset(1, 3,
-                                             self.ten_year_dataset,
+        self.tempSubset = dp.temporal_subset(self.ten_year_dataset,
+                                             1, 3,
                                              average_each_year=True)
         self.assertEqual(self.tempSubset.name, self.ten_year_dataset.name)
         self.assertEqual(self.tempSubset.variable,
@@ -76,8 +77,8 @@ class TestTemporalSubset(unittest.TestCase):
     def test_temporal_subset_equal_start_end_month(self):
         self.dataset_times = np.array([datetime.datetime(year, 1, 1)
                                        for year in range(2000, 2010)])
-        self.tempSubset = dp.temporal_subset(1, 1,
-                                             self.ten_year_dataset,
+        self.tempSubset = dp.temporal_subset(self.ten_year_dataset,
+                                             1, 1,
                                              average_each_year=True)
         np.testing.assert_array_equal(self.dataset_times,
                                       self.tempSubset.times)
@@ -86,27 +87,28 @@ class TestTemporalSubset(unittest.TestCase):
         self.dataset_times = np.array([datetime.datetime(year, month, 1)
                                        for year in range(2000, 2010)
                                        for month in [1, 8, 9, 10, 11, 12]])
-        self.tempSubset = dp.temporal_subset(8, 1, self.ten_year_dataset)
+        self.tempSubset = dp.temporal_subset(self.ten_year_dataset, 8, 1)
         np.testing.assert_array_equal(
             self.dataset_times, self.tempSubset.times)
 
 
 class TestTemporalRebinWithTimeIndex(unittest.TestCase):
+
     def setUp(self):
         self.ten_year_dataset = ten_year_monthly_dataset()
 
     def test_time_dimension_multiple_of_orig_time_dimension(self):
         # ten_year_dataset.times.size is 120
-        nt_avg = self.ten_year_dataset.times.size / 2
+        nt_avg = self.ten_year_dataset.times.size // 2
         # Temporal Rebin to exactly 2 (time) values
         dataset = dp.temporal_rebin_with_time_index(
             self.ten_year_dataset, nt_avg)
         start_time = self.ten_year_dataset.times[0]
         # First month of the middle year
-        middle_element = self.ten_year_dataset.times.size / 2
+        middle_element = self.ten_year_dataset.times.size // 2
         end_time = self.ten_year_dataset.times[middle_element]
         self.assertEqual(dataset.times.size,
-                         self.ten_year_dataset.times.size / nt_avg)
+                         self.ten_year_dataset.times.size // nt_avg)
         np.testing.assert_array_equal(dataset.times, [start_time, end_time])
 
     def test_time_dimension_not_multiple_of_orig_time_dimension(self):
@@ -117,7 +119,7 @@ class TestTemporalRebinWithTimeIndex(unittest.TestCase):
             self.ten_year_dataset, nt_avg)
         new_times = self.ten_year_dataset.times[::11][:-1]
         self.assertEqual(dataset.times.size,
-                         self.ten_year_dataset.times.size / nt_avg)
+                         self.ten_year_dataset.times.size // nt_avg)
         np.testing.assert_array_equal(dataset.times, new_times)
 
     def test_returned_dataset_attributes(self):
@@ -137,6 +139,7 @@ class TestTemporalRebinWithTimeIndex(unittest.TestCase):
 
 
 class TestVariableUnitConversion(unittest.TestCase):
+
     def setUp(self):
         self.ten_year_dataset = ten_year_monthly_dataset()
         self.ten_year_dataset.variable = 'temp'
@@ -199,6 +202,7 @@ class TestVariableUnitConversion(unittest.TestCase):
 
 
 class TestTemporalSlice(unittest.TestCase):
+
     def test_returned_dataset_times(self):
         ''' Tests returned dataset times values '''
         self.ten_year_dataset = ten_year_monthly_dataset()
@@ -206,9 +210,9 @@ class TestTemporalSlice(unittest.TestCase):
         end_index = 4
         dates = np.array([datetime.datetime(2000, month, 1)
                           for month in range(start_index + 1, end_index + 2)])
-        new_dataset = dp.temporal_slice(start_index,
-                                        end_index,
-                                        self.ten_year_dataset)
+        new_dataset = dp.temporal_slice(self.ten_year_dataset,
+                                        start_time = self.ten_year_dataset.times[start_index],
+                                        end_time = self.ten_year_dataset.times[end_index])
         np.testing.assert_array_equal(new_dataset.times, dates)
 
     def test_returned_dataset_values(self):
@@ -217,13 +221,14 @@ class TestTemporalSlice(unittest.TestCase):
         start_index = 1
         end_index = 4
         values = self.ten_year_dataset.values[start_index:end_index + 1]
-        new_dataset = dp.temporal_slice(start_index,
-                                        end_index,
-                                        self.ten_year_dataset)
+        new_dataset = dp.temporal_slice(self.ten_year_dataset,
+                                        start_time = self.ten_year_dataset.times[start_index],
+                                        end_time = self.ten_year_dataset.times[end_index])
         np.testing.assert_array_equal(new_dataset.values, values)
 
 
 class TestEnsemble(unittest.TestCase):
+
     def test_unequal_dataset_shapes(self):
         self.ten_year_dataset = ten_year_monthly_dataset()
         self.two_year_dataset = two_year_daily_dataset()
@@ -396,6 +401,7 @@ class TestSpatialRegrid(unittest.TestCase):
 
 
 class TestNormalizeDatasetDatetimes(unittest.TestCase):
+
     def setUp(self):
         self.monthly_dataset = ten_year_monthly_15th_dataset()
         self.daily_dataset = two_year_daily_2hr_dataset()
@@ -426,31 +432,32 @@ class TestNormalizeDatasetDatetimes(unittest.TestCase):
 
 
 class TestSubset(unittest.TestCase):
+
     def setUp(self):
         self.target_dataset = ten_year_monthly_dataset()
         self.name = 'foo'
 
         self.subregion = ds.Bounds(
-            -81, 81,
-            -161, 161,
-            datetime.datetime(2001, 1, 1),
-            datetime.datetime(2004, 1, 1)
+            lat_min=-81, lat_max=81,
+            lon_min=-161, lon_max=161,
+            start=datetime.datetime(2001, 1, 1),
+            end=datetime.datetime(2004, 1, 1)
         )
         self.non_exact_spatial_subregion = ds.Bounds(
-            -80.25, 80.5,
-            -160.25, 160.5,
-            datetime.datetime(2001, 1, 1),
-            datetime.datetime(2004, 1, 1)
+            lat_min=-80.25, lat_max=80.5,
+            lon_min=-160.25, lon_max=160.5,
+            start=datetime.datetime(2001, 1, 1),
+            end=datetime.datetime(2004, 1, 1)
         )
         self.non_exact_temporal_subregion = ds.Bounds(
-            -80.25, 80.5,
-            -160.25, 160.5,
-            datetime.datetime(2001, 1, 15),
-            datetime.datetime(2004, 2, 15)
+            lat_min=-80.25, lat_max=80.5,
+            lon_min=-160.25, lon_max=160.5,
+            start=datetime.datetime(2001, 1, 15),
+            end=datetime.datetime(2004, 2, 15)
         )
 
     def test_subset(self):
-        subset = dp.subset(self.subregion, self.target_dataset)
+        subset = dp.subset(self.target_dataset, self.subregion)
         self.assertEqual(subset.lats.shape[0], 82)
         self.assertSequenceEqual(list(np.array(range(-81, 82, 2))),
                                  list(subset.lats))
@@ -459,17 +466,17 @@ class TestSubset(unittest.TestCase):
         self.assertEqual(subset.values.shape, (37, 82, 162))
 
     def test_subset_name(self):
-        subset = dp.subset(self.subregion, self.target_dataset)
+        subset = dp.subset(self.target_dataset, self.subregion)
         self.assertEqual(subset.name, self.name)
 
     def test_subset_name_propagation(self):
         subset_name = 'foo_subset_name'
-        subset = dp.subset(self.subregion, self.target_dataset, subset_name)
+        subset = dp.subset(self.target_dataset, self.subregion, subset_name)
         self.assertEqual(subset.name, subset_name)
 
     def test_subset_using_non_exact_spatial_bounds(self):
         index_slices = dp._get_subregion_slice_indices(
-            self.non_exact_spatial_subregion, self.target_dataset)
+            self.target_dataset, self.non_exact_spatial_subregion)
         control_index_slices = {"lat_start": 5,
                                 "lat_end": 84,
                                 "lon_start": 10,
@@ -480,7 +487,7 @@ class TestSubset(unittest.TestCase):
 
     def test_subset_using_non_exact_temporal_bounds(self):
         index_slices = dp._get_subregion_slice_indices(
-            self.non_exact_temporal_subregion, self.target_dataset)
+            self.target_dataset, self.non_exact_temporal_subregion)
         control_index_slices = {"lat_start": 5,
                                 "lat_end": 84,
                                 "lon_start": 10,
@@ -491,10 +498,10 @@ class TestSubset(unittest.TestCase):
 
     def test_subset_without_start_index(self):
         self.subregion = ds.Bounds(
-            -81, 81,
-            -161, 161,
+            lat_min=-81, lat_max=81,
+            lon_min=-161, lon_max=161,
         )
-        subset = dp.subset(self.subregion, self.target_dataset)
+        subset = dp.subset(self.target_dataset, self.subregion)
         times = np.array([datetime.datetime(year, month, 1)
                           for year in range(2000, 2010)
                           for month in range(1, 13)])
@@ -508,6 +515,7 @@ class TestSubset(unittest.TestCase):
 
 
 class TestSafeSubset(unittest.TestCase):
+
     def setUp(self):
         lats = np.array(range(-60, 61, 1))
         lons = np.array(range(-170, 171, 1))
@@ -524,29 +532,29 @@ class TestSafeSubset(unittest.TestCase):
                                          name='foo')
 
         self.spatial_out_of_bounds = ds.Bounds(
-            -165, 165,
-            -180, 180,
-            datetime.datetime(2001, 1, 1),
-            datetime.datetime(2004, 1, 1)
+            lat_min=-65, lat_max=65,
+            lon_min=-180, lon_max=180,
+            start=datetime.datetime(2001, 1, 1),
+            end=datetime.datetime(2004, 1, 1)
         )
 
         self.temporal_out_of_bounds = ds.Bounds(
-            -40, 40,
-            -160.25, 160.5,
-            datetime.datetime(1999, 1, 15),
-            datetime.datetime(2222, 2, 15)
+            lat_min=-40, lat_max=40,
+            lon_min=-160.25, lon_max=160.5,
+            start=datetime.datetime(1999, 1, 15),
+            end=datetime.datetime(2222, 2, 15)
         )
 
         self.everything_out_of_bounds = ds.Bounds(
-            -165, 165,
-            -180, 180,
-            datetime.datetime(1999, 1, 15),
-            datetime.datetime(2222, 2, 15)
+            lat_min=-65, lat_max=65,
+            lon_min=-180, lon_max=180,
+            start=datetime.datetime(1999, 1, 15),
+            end=datetime.datetime(2222, 2, 15)
         )
 
     def test_partial_spatial_overlap(self):
         '''Ensure that safe_subset can handle out of bounds spatial values'''
-        ds = dp.safe_subset(self.spatial_out_of_bounds, self.target_dataset)
+        ds = dp.safe_subset(self.target_dataset, self.spatial_out_of_bounds)
         spatial_bounds = ds.spatial_boundaries()
         self.assertEquals(spatial_bounds[0], -60)
         self.assertEquals(spatial_bounds[1], 60)
@@ -555,8 +563,8 @@ class TestSafeSubset(unittest.TestCase):
 
     def test_partial_temporal_overlap(self):
         '''Ensure that safe_subset can handle out of bounds temporal values'''
-        ds = dp.safe_subset(self.temporal_out_of_bounds, self.target_dataset)
-        temporal_bounds = ds.time_range()
+        ds = dp.safe_subset(self.target_dataset, self.temporal_out_of_bounds)
+        temporal_bounds = ds.temporal_boundaries()
         start = datetime.datetime(2000, 1, 1)
         end = datetime.datetime(2009, 12, 1)
 
@@ -564,9 +572,9 @@ class TestSafeSubset(unittest.TestCase):
         self.assertEquals(temporal_bounds[1], end)
 
     def test_entire_bounds_overlap(self):
-        ds = dp.safe_subset(self.everything_out_of_bounds, self.target_dataset)
+        ds = dp.safe_subset(self.target_dataset, self.everything_out_of_bounds)
         spatial_bounds = ds.spatial_boundaries()
-        temporal_bounds = ds.time_range()
+        temporal_bounds = ds.temporal_boundaries()
         start = datetime.datetime(2000, 1, 1)
         end = datetime.datetime(2009, 12, 1)
 
@@ -579,50 +587,52 @@ class TestSafeSubset(unittest.TestCase):
 
 
 class TestFailingSubset(unittest.TestCase):
+
     def setUp(self):
         self.target_dataset = ten_year_monthly_dataset()
         self.target_dataset.lats = np.array(range(-89, 88, 2))
         self.target_dataset.lons = np.array(range(-179, 178, 2))
 
         self.subregion = ds.Bounds(
-            -81, 81,
-            -161, 161,
-            datetime.datetime(2001, 1, 1),
-            datetime.datetime(2004, 1, 1)
+            lat_min=-81, lat_max=81,
+            lon_min=-161, lon_max=161,
+            start=datetime.datetime(2001, 1, 1),
+            end=datetime.datetime(2004, 1, 1)
         )
 
     def test_out_of_dataset_bounds_lat_min(self):
-        self.subregion.lat_min = -90
+        self.subregion.lat_max = -90
         with self.assertRaises(ValueError):
-            dp.subset(self.subregion, self.target_dataset)
+            dp.subset(self.target_dataset, self.subregion)
 
     def test_out_of_dataset_bounds_lat_max(self):
-        self.subregion.lat_max = 90
+        self.subregion.lat_min = 90
         with self.assertRaises(ValueError):
-            dp.subset(self.subregion, self.target_dataset)
+            dp.subset(self.target_dataset, self.subregion)
 
     def test_out_of_dataset_bounds_lon_min(self):
-        self.subregion.lon_min = -180
+        self.subregion.lon_max = -180
         with self.assertRaises(ValueError):
-            dp.subset(self.subregion, self.target_dataset)
+            dp.subset(self.target_dataset, self.subregion)
 
     def test_out_of_dataset_bounds_lon_max(self):
-        self.subregion.lon_max = 180
+        self.subregion.lon_min = 180
         with self.assertRaises(ValueError):
-            dp.subset(self.subregion, self.target_dataset)
+            dp.subset(self.target_dataset, self.subregion)
 
     def test_out_of_dataset_bounds_start(self):
         self.subregion.start = datetime.datetime(1999, 1, 1)
         with self.assertRaises(ValueError):
-            dp.subset(self.subregion, self.target_dataset)
+            dp.subset(self.target_dataset, self.subregion)
 
     def test_out_of_dataset_bounds_end(self):
         self.subregion.end = datetime.datetime(2011, 1, 1)
         with self.assertRaises(ValueError):
-            dp.subset(self.subregion, self.target_dataset)
+            dp.subset(self.target_dataset, self.subregion)
 
 
 class TestNetCDFWrite(unittest.TestCase):
+
     def setUp(self):
         self.ds = ten_year_monthly_dataset()
         self.ds_2d = ten_year_monthly_dataset(latlon2d=True)
