@@ -18,13 +18,18 @@
 ''''''
 
 import sys
+import os
 import datetime as dt
 import numpy as np
 import numpy.ma as ma
 
-from mpl_toolkits.basemap import shiftgrid
+from mpl_toolkits.basemap import shiftgrid, Basemap
+from matplotlib.path import Path
 from dateutil.relativedelta import relativedelta
-from netCDF4 import num2date
+from netCDF4 import num2date, date2num
+import scipy.interpolate as interpolate
+import scipy.stats as stats
+from scipy.ndimage import map_coordinates
 
 
 def decode_time_values(dataset, time_var_name):
@@ -65,6 +70,9 @@ def decode_time_values(dataset, time_var_name):
 
         times = num2date(
             time_data[:], units=time_format, calendar=times_calendar)
+        if times_calendar == '360_day':
+            for it, t in enumerate(times):
+                times[it] = dt.datetime.strptime(str(t), '%Y-%m-%d 00:00:00')
     return times
 
 
@@ -264,7 +272,12 @@ def reshape_monthly_to_annually(dataset):
     values = dataset.values[:]
     data_shape = values.shape
     num_total_month = data_shape[0]
-    num_year = num_total_month / 12
+    num_year = num_total_month // 12
+    if num_total_month % 12 != 0:
+        raise ValueError("Number of months in dataset ({}) does not "
+                         "divide evenly into {} year(s)."
+                         .format(num_total_month, num_year))
+
     num_month = 12
     year_month_shape = num_year, num_month
     lat_lon_shape = data_shape[1:]
@@ -360,7 +373,7 @@ def calc_time_series(dataset):
     '''
 
     t_series = []
-    for t in xrange(dataset.values.shape[0]):
+    for t in range(dataset.values.shape[0]):
         t_series.append(dataset.values[t, :, :].mean())
 
     return t_series
@@ -374,20 +387,23 @@ def get_temporal_overlap(dataset_array):
     start_time = []
     end_time = []
     for dataset in dataset_array:
-        start_time.append(dataset.time_range()[0])
-        end_time.append(dataset.time_range()[1])
+        start_time.append(dataset.temporal_boundaries()[0])
+        end_time.append(dataset.temporal_boundaries()[1])
 
     return np.max(start_time), np.min(end_time)
 
 
 def calc_subregion_area_mean_and_std(dataset_array, subregions):
-    ''' Calculate area mean and standard deviation values for a given
+    ''' Calculate area mean and standard deviation values for a given \
         subregions using datasets on common grid points
-    :param dataset_array: An array of OCW Dataset Objects
-    :type list:
-    :param subregions: list of subregions
+
+    :param dataset_array: An array of OCW Dataset Objects \
+    :type list: :mod:'list'
+
+    :param subregions: list of subregions \
     :type subregions: :class:`numpy.ma.array`
-    :returns: area averaged time series for the dataset of shape
+
+    :returns: area averaged time series for the dataset of shape \
               (ntime, nsubregion)
     '''
 
@@ -445,3 +461,233 @@ def calc_area_weighted_spatial_average(dataset, area_weight=False):
             spatial_average[it] = ma.average(dataset.values[it, :])
 
     return spatial_average
+
+
+def shapefile_boundary(boundary_type, region_names):
+    '''
+    :param boundary_type: The type of spatial subset boundary
+    :type boundary_type: :mod:'string'
+
+    :param region_names: An array of regions for spatial subset
+    :type region_names: :mod:'list'
+    '''
+    # Read the shapefile
+    map_read = Basemap()
+    regions = []
+    shapefile_dir = os.sep.join([os.path.dirname(__file__), 'shape'])
+    map_read.readshapefile(os.path.join(shapefile_dir, boundary_type),
+                           boundary_type, drawbounds=False)
+    # Note: The shapefile may contain countries or states with latin letters.
+    # Hence, the output of readshapefile can be a mix of ascii and unicode
+    # strings. Because python 2 and 3 treat them differently, we must
+    # explicitly check.
+    if boundary_type == 'us_states':
+        for region_name in region_names:
+            region_name = _force_unicode(region_name)
+            for iregion, region_info in enumerate(map_read.us_states_info):
+                state1 = _force_unicode(region_info['st'], 'latin-1')
+                state2 = _force_unicode(region_info['state'], 'latin-1')
+                if state1 == region_name or state2 == region_name:
+                    regions.append(np.array(map_read.us_states[iregion]))
+    elif boundary_type == 'countries':
+        for region_name in region_names:
+            region_name = _force_unicode(region_name)
+            for iregion, region_info in enumerate(map_read.countries_info):
+                country = _force_unicode(region_info['COUNTRY'], 'latin-1')
+                if (country.replace(" ", "").lower() ==
+                    region_name.replace(" ", "").lower()):
+                    regions.append(np.array(map_read.countries[iregion]))
+    return regions
+
+
+def CORDEX_boundary(domain_name):
+    '''
+    :param domain_name: CORDEX domain name (http://www.cordex.org/)
+    :type domain_name: :mod:'string'
+    '''
+    if domain_name == 'southamerica':
+        return -57.61, 18.50, 254.28 - 360., 343.02 - 360.
+    elif domain_name == 'centralamerica':
+        return -19.46, 34.83, 235.74 - 360., 337.78 - 360.
+    elif domain_name == 'northamerica':
+        return 12.55, 75.88, 189.26 - 360., 336.74 - 360.
+    elif domain_name == 'europe':
+        return 22.20, 71.84, 338.23 - 360., 64.4
+    elif domain_name == 'africa':
+        return -45.76, 42.24, 335.36 - 360., 60.28
+    elif domain_name == 'southasia':
+        return -15.23, 45.07, 19.88, 115.55
+    elif domain_name == 'eastasia':
+        return -0.10, 61.90, 51.59, 179.99
+    elif domain_name == 'centralasia':
+        return 18.34, 69.37, 11.05, 139.13
+    elif domain_name == 'australasia':
+        return -52.36, 12.21, 89.25, 179.99
+    elif domain_name == 'antartica':
+        return -89.48, -56.00, -179.00, 179.00
+    elif domain_name == 'artic':
+        return 46.06, 89.50, -179.00, 179.00
+    elif domain_name == 'mediterranean':
+        return 25.63, 56.66, 339.79 - 360.00, 50.85
+    elif domain_name == 'middleeastnorthafrica':
+        return -7.00, 45.00, 333.00 - 360.00, 76.00
+    elif domain_name == 'southeastasia':
+        return -15.14, 27.26, 89.26, 146.96
+    else:
+        err = "Invalid CORDEX domain name"
+        raise ValueError(err)
+
+
+def mask_using_shapefile_info(lons, lats, masked_regions, extract=True):
+    if lons.ndim == 2 and lats.ndim == 2:
+        lons_2d = lons
+        lats_2d = lats
+    else:
+        lons_2d, lats_2d = np.meshgrid(lons, lats)
+
+    points = np.array((lons_2d.flatten(), lats_2d.flatten())).T
+    for iregion, region in enumerate(masked_regions):
+        mpath = Path(region)
+        mask0 = mpath.contains_points(points).reshape(lons_2d.shape)
+        if iregion == 0:
+            mask = mask0
+        else:
+            mask = mask | mask0
+    if extract:
+        mask = np.invert(mask)
+    return mask
+
+
+def regrid_spatial_mask(target_lon, target_lat, mask_lon, mask_lat, mask_var,
+                        user_mask_values, extract=True):
+    target_lons, target_lats = convert_lat_lon_2d_array(target_lon, target_lat)
+    mask_lons, mask_lats = convert_lat_lon_2d_array(mask_lon, mask_lat)
+
+    mask_var_regridded = interpolate.griddata((mask_lons.flatten(), mask_lats.flatten()),
+                                              mask_var.flatten(),
+                                              (target_lons.flatten(),
+                                               target_lats.flatten()),
+                                              method='nearest',
+                                              fill_value=-9999.).reshape(target_lons.shape)
+
+    mask_outside = ma.masked_equal(mask_var_regridded, -9999.).mask
+    values_original = ma.array(mask_var)
+    for shift in (-1, 1):
+        for axis in (0, 1):
+            q_shifted = np.roll(values_original, shift=shift, axis=axis)
+            idx = ~q_shifted.mask * values_original.mask
+            values_original.data[idx] = q_shifted[idx]
+    # Make a masking map using nearest neighbour interpolation -use this to
+    # determine locations with MDI and mask these
+    qmdi = np.zeros_like(values_original)
+    qmdi[values_original.mask == True] = 1.
+    qmdi[values_original.mask == False] = 0.
+    qmdi_r = map_coordinates(qmdi, [target_lats.flatten(
+    ), target_lons.flatten()], order=1).reshape(target_lons.shape)
+    mdimask = (qmdi_r != 0.0)
+
+    for value in user_mask_values:
+        mask_var_regridded = ma.masked_equal(mask_var_regridded, value)
+
+    if extract:
+        mask_out = np.invert(mask_var_regridded.mask | mdimask)
+        return mask_out | mask_outside
+    else:
+        mask_out = mask_var_regridded.mask | mdimask
+        return mask_out | mask_outside
+
+
+def propagate_spatial_mask_over_time(data_array, mask):
+    if data_array.ndim == 3 and mask.ndim == 2:
+        nt = data_array.shape[0]
+        new_data_array = ma.zeros(data_array.shape)
+        if isinstance(data_array.mask, bool):
+            new_mask = mask
+            for it in np.arange(nt):
+                new_data_array[it,:] = ma.array(data_array[it,:],
+                                            mask=new_mask)
+        else:
+            for it in np.arange(nt):
+                new_mask = data_array[it, :].mask | mask
+                new_data_array[it,:] = ma.array(data_array[it,:],
+                                            mask=new_mask)
+
+        return new_data_array
+
+
+def convert_lat_lon_2d_array(lon, lat):
+    if lon.ndim == 1 and lat.ndim == 1:
+        return np.meshgrid(lon, lat)
+    if lon.ndim == 2 and lat.ndim == 2:
+        return lon, lat
+
+def _force_unicode(s, encoding='utf-8'):
+    '''
+    If the input is bytes, convert to unicode, otherwise return the input
+    '''
+    if hasattr(s, 'decode'):
+        s = s.decode(encoding=encoding)
+        
+    return s
+
+def calculate_temporal_trends(dataset):
+    ''' Calculate temporal trends in dataset.values
+    :param dataset: The dataset from which time values should be extracted.
+    :type dataset: :class:`dataset.Dataset'
+
+    :returns: Arrays of the temporal trend and standard error
+    :rtype: :class:`numpy.ma.core.MaskedArray`
+    '''
+
+    nt, ny, nx = dataset.values.shape
+    x = np.arange(nt)
+
+    trend = np.zeros([ny, nx])-999.
+    slope_err = np.zeros([ny, nx])-999.
+    for iy in np.arange(ny):
+        for ix in np.arange(nx):
+            if dataset.values[:,iy,ix].count() == nt:
+                trend[iy,ix], slope_err[iy,ix] = calculate_temporal_trend_of_time_series(
+                                               x, dataset.values[:,iy,ix])
+    
+    return ma.masked_equal(trend, -999.), ma.masked_equal(slope_err, -999.)
+
+def calculate_ensemble_temporal_trends(timeseries_array, number_of_samples=1000):
+    ''' Calculate temporal trends in an ensemble of time series
+    :param timeseries_array: Two dimensional array. 1st index: model, 2nd index: time.
+    :type timeseries_array: :class:`numpy.ndarray'
+
+    :param sampling: A list whose elements are one-dimensional numpy arrays
+    :type timeseries_array: :class:`list'
+
+    :returns: temporal trend and estimated error from bootstrapping
+    :rtype: :float:`float','float'
+    '''
+   
+    nmodels, nt = timeseries_array.shape
+    x = np.arange(nt)
+    sampled_trend = np.zeros(number_of_samples)
+    ensemble_trend, _ = calculate_temporal_trend_of_time_series(
+                        x, np.mean(timeseries_array, axis=0))
+    
+    for isample in np.arange(number_of_samples):
+        index = np.random.choice(nmodels, size=nmodels, replace=True)
+        random_ensemble = np.mean(timeseries_array[index, :], axis=0)
+        sampled_trend[isample], _ = calculate_temporal_trend_of_time_series(
+                                    x, random_ensemble)
+    return ensemble_trend, np.std(sampled_trend, ddof=1)
+     
+
+def calculate_temporal_trend_of_time_series(x,y):
+    ''' Calculate least-square trends (a) in y = ax+b and a's standard error
+    :param x: time series
+    :type x: :class:`numpy.ndarray'
+
+    :param x: time series
+    :type x: :class:`numpy.ndarray'
+
+    :returns: temporal trend and standard error
+    :rtype: :float:`float','float'
+    '''
+    slope, intercept, r_value, p_value, std_err = stats.linregress(x,y)
+    return slope, std_err
